********************************************************
*    CALCULATOR <CALC>                                 *
********************************************************   
*  Converts character STRING(1..80)*1 of the form:     *
*                                                      *
*     Express1, Express2, ... ,ExpressN    {! comment} *
*                                                      *
*  to the value:                                       *
*                                                      *
*     Vec(1)  , Vec(2)  , ... , Vec(N)                 *
*                                                      *
*  Expressions can include any of the following opera- *
*  tions: /, *, +, - as well as any nested paranthesi- *
*  ses expressions.                                    *
*                                                      *
*  Special functions allowed:                          *
*                                                      *
*  >  COS,SIN,LOG,EXP,TAN,SQRT,CBRT=X**(1/3)           * 
*                                                      *
*  Nesting in special functions like COS(TAN(...)) is  *
*  not allowed!                                        *
*                                                      *
*  Special constants allowed:                          *
*                                                      *
*  >  PI=3.141592653589793D0                           *
*                                                      *
*  Degrees "**" (equivalently "^") are allowed with    *
*  the format                                          *
*                                                      *
*  >  (...)**(...) or equivalently (...)^(...)         *
*                                                      *
*  where the expressions in brakets are either simple  *
*  formulae comtaining  /, *, +, -, or the numbers.    *
*  Expamples: 2^(1/3), pi**0.672, (0.534+8.21/3)^(1/8) *
*                                                      *
*  Any degree of the special function like cos(...)^2  *
*  is not allowed! But one can type cos(pi^2) since    *
*  any expression containing degrees are translated    *
*  first.                                              *
*                                                      *
*  No case sensitivity (COS=cOs, PI=pi, ...) assumed.  *
*                                                      *
*  Format for numerical constants:                     *
*                                                      *
*  >  1 1. 1.0 1.e0 1.d0                               * 
*                                                      *
*  Degrees in the constants like 1.e+00, 1.e-00        *
*  do not allowed! (Sign + or - will be wrongly        *
*  processed.)                                         *
*                                                      *
*  Examples:                                           *
*                                                      *
*  >  Express := -1.d0+cos(pi/(1+1.0))/sqrt(((4.e0)))  *
*             := -1.0                                  *
*                                                      *
*  Notes: in the expression containing only the        *
*  operations /, *, +, -, the first operation perform- *
*  ed is '/', the second is '*', the third is '-'      *
*  and the last is '+'. This consequence can be chang- *
*  ed using paranthesises relations.                   *
*                                                      *
*  Created: Nov. 1995, S. Savrasov.                    *
*  Revised: Mar. 1996.                                 *
********************************************************
C	DOUBLE PRECISION ANSWER
C	CHARACTER STRING(80)*1
C	PRINT*,'<CALC> Version 1.1, Nov.1995, READY --->'
C  10	READ(5,1010)STRING
C1010	FORMAT(80A1)
C	IF(STRING(1).EQ.'q'.OR.STRING(1).EQ.'Q')STOP
C	CALL COMPVEC(STRING,ANSWER,1)
C	PRINT*,ANSWER
C	GOTO10
C	END


	SUBROUTINE COMPVEC(STRING,VEC,N)
********************************************************
*  Recognizes vector in a character string.            *
********************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION VEC(N),NCOMMA(0:80)
	CHARACTER STRING(80)*1,CVEC(80)*1,BLANCK*1,TABULS*1
	BLANCK=' '
	TABULS='	'
	IERR=0
C  REMOVE COMMENTS
	ICOM=81
	DO 10 I=1,80
	IF(STRING(I).EQ.'!')ICOM=I
	IF(STRING(I).EQ.'!')GOTO20
  10	CONTINUE
  20	DO 30 I=ICOM,80
	STRING(I)=BLANCK
  30	CONTINUE
C  FIND COMMAS
	ICOMMA=0
	NCOMMA(0)=0
	DO 40 I=1,80
	IF(STRING(I).EQ.',')THEN
	ICOMMA=ICOMMA+1
	NCOMMA(ICOMMA)=I
	ENDIF
  40	CONTINUE
	NCOMMA(ICOMMA+1)=80
	IF(ICOMMA.NE.N-1)STOP'<CALC>:: # of "," mismatch in the 
     &  expression translated'
C  FIND VECTOR COORDINATES SEPARATED BY COMMAS
	DO 60 ICOORD=1,N
	ISTA=NCOMMA(ICOORD-1)+1
	IEND=NCOMMA(ICOORD)-1
	LENGHT=0
	DO 50 I=ISTA,IEND
	LENGHT=LENGHT+1
	CVEC(LENGHT)=STRING(I)
  50	CONTINUE
	IF(LENGHT.LE.0)STOP'<CALC>:: Wrong syntax in STRING'
C  COMPUTE EXPRESSION
	CALL COMPILE(CVEC(1),LENGHT,VEC(ICOORD))
  60	CONTINUE
	END



	SUBROUTINE COMPILE(STRING,LENGHT,COORD)
***************************************************************
*  Compiles a single string, recognizes special functions and *
*  +,-,*,/,(,)                                                *
***************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	CHARACTER STRING(80)*1,EXPRESS(80)*30,BLANCK*1,VALUE*30
     &    ,LBLANCK*30
	BLANCK=' '
	LBLANCK='                              '
C  CONVERT TO LOWER CASE
	CALL CONVERT(STRING,LENGHT)
C  CHECK FOR THE CORECT NUMBER OF PARANTHESISES
   	CALL CHECKPA(STRING,LENGHT)
C  INITIALIZE FILE "EXPRESS"
	DO 10 I=1,LENGHT
	EXPRESS(I)='                    '
  10	CONTINUE
C  TRANSFER DEGREES FIRST
	DO 15 I=1,LENGHT
C  PI
	IF(I.LE.LENGHT-1)THEN
	IF(STRING(I+0).EQ.'p'.AND.
     &	   STRING(I+1).EQ.'i')THEN
        EXPRESS(I)(:19)='3.141592653589793d0'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	ENDIF
	ENDIF
C  DEGREE "**"
	IF(I.LE.LENGHT-1)THEN
	IF(STRING(I+0).EQ.'*'.AND.
     &	   STRING(I+1).EQ.'*')THEN
        EXPRESS(I)(:2)='**'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	ENDIF
	ENDIF
C  THE SAME DEGREE "^"
	IF(I.LE.LENGHT)THEN
	IF(STRING(I).EQ.'^')THEN
        EXPRESS(I)(:2)='**'
	STRING(I)=BLANCK
	ENDIF
	ENDIF
  15	CONTINUE
C  TRANSFER SIMPLE OPERATIONS TO "EXPRESS"
	DO 20 I=1,LENGHT
	IF(STRING(I).EQ.'+'.OR.
     &	   STRING(I).EQ.'-'.OR.
     &	   STRING(I).EQ.'*'.OR.
     &	   STRING(I).EQ.'/'.OR.
     &	   STRING(I).EQ.'('.OR.
     &	   STRING(I).EQ.')')THEN
	EXPRESS(I)(:1)=STRING(I)
	STRING(I)=BLANCK
	ENDIF
  20	CONTINUE
C  TRANSFER SPECIAL FUNCTIONS
	DO 30 I=1,LENGHT
C  COS
	IF(I.LE.LENGHT-2)THEN
	IF(STRING(I+0).EQ.'c'.AND.
     &	   STRING(I+1).EQ.'o'.AND.
     &	   STRING(I+2).EQ.'s')THEN
	EXPRESS(I)(:4)='cos1'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	STRING(I+2)=BLANCK
	ENDIF
	ENDIF
C  SIN
	IF(I.LE.LENGHT-2)THEN
	IF(STRING(I+0).EQ.'s'.AND.
     &	   STRING(I+1).EQ.'i'.AND.
     &	   STRING(I+2).EQ.'n')THEN
	EXPRESS(I)(:4)='sin1'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	STRING(I+2)=BLANCK
	ENDIF
	ENDIF
C  EXP
	IF(I.LE.LENGHT-2)THEN
	IF(STRING(I+0).EQ.'e'.AND.
     &	   STRING(I+1).EQ.'x'.AND.
     &	   STRING(I+2).EQ.'p')THEN
	EXPRESS(I)(:4)='exp1'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	STRING(I+2)=BLANCK
	ENDIF
	ENDIF
C  LOG
	IF(I.LE.LENGHT-2)THEN
	IF(STRING(I+0).EQ.'l'.AND.
     &	   STRING(I+1).EQ.'o'.AND.
     &	   STRING(I+2).EQ.'g')THEN
	EXPRESS(I)(:4)='log1'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	STRING(I+2)=BLANCK
	ENDIF
	ENDIF
C  TAN
	IF(I.LE.LENGHT-2)THEN
	IF(STRING(I+0).EQ.'t'.AND.
     &	   STRING(I+1).EQ.'a'.AND.
     &	   STRING(I+2).EQ.'n')THEN
	EXPRESS(I)(:4)='tan1'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	STRING(I+2)=BLANCK
	ENDIF
	ENDIF
C  SQRT
	IF(I.LE.LENGHT-3)THEN
	IF(STRING(I+0).EQ.'s'.AND.
     &	   STRING(I+1).EQ.'q'.AND.
     &	   STRING(I+2).EQ.'r'.AND.
     &	   STRING(I+3).EQ.'t')THEN
	EXPRESS(I)(:4)='sqrt'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	STRING(I+2)=BLANCK
	STRING(I+3)=BLANCK
	ENDIF
	ENDIF
C  CBRT - Cubic root
	IF(I.LE.LENGHT-3)THEN
	IF(STRING(I+0).EQ.'c'.AND.
     &	   STRING(I+1).EQ.'b'.AND.
     &	   STRING(I+2).EQ.'r'.AND.
     &	   STRING(I+3).EQ.'t')THEN
	EXPRESS(I)(:4)='cbrt'
	STRING(I+0)=BLANCK
	STRING(I+1)=BLANCK
	STRING(I+2)=BLANCK
	STRING(I+3)=BLANCK
	ENDIF
	ENDIF
  30	CONTINUE
C  CONVERT NUMBERS TO DOUBLE PRECISION
	DO 60 I=1,LENGHT
	IF(STRING(I).EQ.BLANCK)GOTO60
	VALUE='                    '
	ICAT=0
	DO 40 J=I,LENGHT
	IF(STRING(J).NE.BLANCK)THEN
	ICAT=ICAT+1
	IF(ICAT.EQ.1)VALUE(:1)=STRING(J)
	IF(ICAT.GT.1)VALUE(:ICAT)=VALUE(:ICAT-1)//STRING(J)
	STRING(J)=BLANCK
	IF(ICAT.GT.30)STOP'<CALC>:: Too many numbers in the expression'
	ELSEIF(STRING(J).EQ.BLANCK)THEN
	GOTO50
	ENDIF
  40	CONTINUE 
  50	READ(VALUE,*)ANUM
	WRITE(EXPRESS(I),*)ANUM
  60	CONTINUE
C  CHECK FOR UNRECOGNIZED SYMBOLS
	IERR=0
	DO 70 I=1,LENGHT
	IF(STRING(I).NE.BLANCK)THEN
	IERR=4
	PRINT*,'<CALC>:: Unrecognized symbol ',STRING(I)
	ENDIF
  70	CONTINUE
	IF(IERR.NE.0)STOP
C  REMOVE EMPTY LINES
	CALL REMBL(EXPRESS,LENGHT,NEWL)
	LENGHT=NEWL
C  COMPUTE DEGREES FIRST
	DO 75 I=1,LENGHT
	IF(EXPRESS(I)(:2).EQ.'**')THEN
	IF(I.EQ.1)STOP'<CALC>:: Syntax error in power'
C  POWER
	IF(EXPRESS(I+1).EQ.'(')THEN
	CALL SFUNBRA(EXPRESS,I+1,IE)
	CALL CALCBRA(EXPRESS,I+1,IE)
	READ(EXPRESS(I+1),*)POWER
	ELSE
	READ(EXPRESS(I+1),*)POWER
	ENDIF
	EXPRESS(I+1)=LBLANCK
C  NUMBER
	IF(EXPRESS(I-1).EQ.')')THEN
	CALL DFUNBRA(EXPRESS,I-1,IS)
	CALL CALCBRA(EXPRESS,IS,I-1)
	READ(EXPRESS(IS),*)ANUM
	EXPRESS(IS)=LBLANCK
	ELSE
	READ(EXPRESS(I-1),*)ANUM
	EXPRESS(I-1)=LBLANCK
	ENDIF
C  ANSWER
	ANSWER=ANUM**POWER
	WRITE(EXPRESS(I),*)ANSWER
	ENDIF
  75	CONTINUE
C  REMOVE EMPTY LINES
	CALL REMBL(EXPRESS,LENGHT,NEWL)
	LENGHT=NEWL
C  COMPUTE SPECIAL FUNCTIONS
	DO 80 I=1,LENGHT
C  COS
	IF(EXPRESS(I)(:4).EQ.'cos1')THEN
	CALL SFUNBRA(EXPRESS,I+1,IE)
	CALL CALCBRA(EXPRESS,I+1,IE)
	READ(EXPRESS(I+1),*)ANUM
	FNUM=COS(ANUM)
	WRITE(EXPRESS(I),*)FNUM
	EXPRESS(I+1)=LBLANCK
	ENDIF
C  SIN
	IF(EXPRESS(I)(:4).EQ.'sin1')THEN
	CALL SFUNBRA(EXPRESS,I+1,IE)
	CALL CALCBRA(EXPRESS,I+1,IE)
	READ(EXPRESS(I+1),*)ANUM
	FNUM=SIN(ANUM)
	WRITE(EXPRESS(I),*)FNUM
	EXPRESS(I+1)=LBLANCK
	ENDIF
C  TAN
	IF(EXPRESS(I)(:4).EQ.'tan1')THEN
	CALL SFUNBRA(EXPRESS,I+1,IE)
	CALL CALCBRA(EXPRESS,I+1,IE)
	READ(EXPRESS(I+1),*)ANUM
	FNUM=TAN(ANUM)
	WRITE(EXPRESS(I),*)FNUM
	EXPRESS(I+1)=LBLANCK
	ENDIF
C  LOG
	IF(EXPRESS(I)(:4).EQ.'log1')THEN
	CALL SFUNBRA(EXPRESS,I+1,IE)
	CALL CALCBRA(EXPRESS,I+1,IE)
	READ(EXPRESS(I+1),*)ANUM
	FNUM=LOG(ANUM)
	WRITE(EXPRESS(I),*)FNUM
	EXPRESS(I+1)=LBLANCK
	ENDIF
C  EXP
	IF(EXPRESS(I)(:4).EQ.'exp1')THEN
	CALL SFUNBRA(EXPRESS,I+1,IE)
	CALL CALCBRA(EXPRESS,I+1,IE)
	READ(EXPRESS(I+1),*)ANUM
	FNUM=EXP(ANUM)
	WRITE(EXPRESS(I),*)FNUM
	EXPRESS(I+1)=LBLANCK
	ENDIF
C  SQRT
	IF(EXPRESS(I)(:4).EQ.'sqrt')THEN
	CALL SFUNBRA(EXPRESS,I+1,IE)
	CALL CALCBRA(EXPRESS,I+1,IE)
	READ(EXPRESS(I+1),*)ANUM
	FNUM=SQRT(ANUM)
	WRITE(EXPRESS(I),*)FNUM
	EXPRESS(I+1)=LBLANCK
	ENDIF
C  CBRT
	IF(EXPRESS(I)(:4).EQ.'cbrt')THEN
	CALL SFUNBRA(EXPRESS,I+1,IE)
	CALL CALCBRA(EXPRESS,I+1,IE)
	READ(EXPRESS(I+1),*)ANUM
	FNUM=ANUM**(1.D0/3.D0)
	WRITE(EXPRESS(I),*)FNUM
	EXPRESS(I+1)=LBLANCK
	ENDIF
  80	CONTINUE
C  REMOVE EMPTY LINES
	CALL REMBL(EXPRESS,LENGHT,NEWL)
	LENGHT=NEWL
C  COMPUTE FINAL VALUE ``COORD``
	CALL CALCBRA(EXPRESS,1,LENGHT)
	READ(EXPRESS(1),*)COORD
	END


	SUBROUTINE CALCBRA(EXPRESS,NS,NF)
*****************************************************
*  Computes expression containing brakets and simple*
*  operations like * / + -.                         *
*****************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	CHARACTER EXPRESS(80)*30,WORK(80)*30,LBLANCK*30
	LBLANCK='                              '
C  COPY CONTENTS TO THE WORKFILE
	LENGHT=0
	DO 10 I=NS,NF
	LENGHT=LENGHT+1
	WORK(LENGHT)=EXPRESS(I)
	EXPRESS(I)=LBLANCK
  10	CONTINUE
C  *** COMPILE WORKFILE ***
C  UNDERSTAND NESTING
	ILEFT=0
	IRIGH=0
	DO 20 I=1,LENGHT
	IF(WORK(I).EQ.'(')ILEFT=ILEFT+1
	IF(WORK(I).EQ.')')IRIGH=IRIGH+1
  20	CONTINUE
	IF(ILEFT.NE.IRIGH)PRINT*,'<CALC>:: Sintax error'
	IF(ILEFT.NE.IRIGH)STOP
	NEST=ILEFT
	DO 50 INEST=1,NEST
C  SEARCH FOR THE '')'' 
	DO 40 I=1,LENGHT
	IF(WORK(I).EQ.')')THEN
	WORK(I)=LBLANCK
C  SEARCH FOR THE ''('' 
	DO 30 J=I,1,-1
	IF(WORK(J).EQ.'(')THEN
	WORK(J)=LBLANCK
C  PROCESS
	CALL CALCEXP(WORK,J+1,I-1)
	ENDIF
  30	CONTINUE
	ENDIF
  40	CONTINUE
  50	CONTINUE
	CALL CALCEXP(WORK,1,LENGHT)
C  REMOVE BLANCKS
	CALL REMBL(WORK,LENGHT,NEWL)
	LENGHT=NEWL
C  CHECK OUT
	IF(LENGHT.NE.1)STOP'<CALC>:: Wrong compilation of ( )'
C  RETURN
	EXPRESS(NS)=WORK(1)
	END


	SUBROUTINE CALCEXP(WORK,NS,NF)
*****************************************************
*  Computes simple expression with no brakets.      *
*****************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	CHARACTER WORK(80)*30,WRK(80)*30,LBLANCK*30
	LBLANCK='                              '
C  COPY CONTENTS TO THE WORKFILE
	LENGHT=0
	DO 10 I=NS,NF
	LENGHT=LENGHT+1
	WRK(LENGHT)=WORK(I)
	WORK(I)=LBLANCK
  10	CONTINUE
C  REMOVE BLANCKS
	CALL REMBL(WRK,LENGHT,NEWL)
	LENGHT=NEWL
C  ESTIMATE NUMBER OF OPERATIONS
	NDIV=0
	NMUL=0
	NPLS=0
	NMNS=0
	DO 20 I=1,LENGHT
	IF(WRK(I)(:1).EQ.'/')NDIV=NDIV+1
	IF(WRK(I)(:1).EQ.'*')NMUL=NMUL+1
	IF(WRK(I)(:1).EQ.'+')NPLS=NPLS+1
	IF(WRK(I)(:1).EQ.'-')NMNS=NMNS+1
  20	CONTINUE
C  DO OVER DIVISIONS
	DO 50 IDIV=1,NDIV
	DO 30 I=1,LENGHT
	IF(WRK(I)(:1).EQ.'/')THEN
        IF(I.EQ.1)STOP'<CALCEXP>:: Error in division 1'
        IF(I.EQ.LENGHT)STOP'<CALCEXP>:: Error in division 2'
	READ(WRK(I-1),*)A1
	READ(WRK(I+1),*)A2
	IF(ABS(A2).LE.1.D-14)STOP'<CALCEXP>:: Division to zero'
	ADIV=A1/A2
	WRITE(WRK(I-1),*)ADIV
	WRK(I+0)=LBLANCK
	WRK(I+1)=LBLANCK
	GOTO40
	ENDIF
  30	CONTINUE
C  REMOVE BLANCKS
  40	CALL REMBL(WRK,LENGHT,NEWL)
	LENGHT=NEWL
  50	CONTINUE
C  DO OVER MULTIPLICATIONS
	DO 80 IMUL=1,NMUL
	DO 60 I=1,LENGHT
	IF(WRK(I)(:1).EQ.'*')THEN
        IF(I.EQ.1)STOP'<CALCEXP>:: Error in multiplication 1'
        IF(I.EQ.LENGHT)STOP'<CALCEXP>:: Error in multiplication 2'
	READ(WRK(I-1),*)A1
	READ(WRK(I+1),*)A2
	AMUL=A1*A2
	WRITE(WRK(I-1),*)AMUL
	WRK(I+0)=LBLANCK
	WRK(I+1)=LBLANCK
	GOTO70
	ENDIF
  60	CONTINUE
C  REMOVE BLANCKS
  70	CALL REMBL(WRK,LENGHT,NEWL)
	LENGHT=NEWL
  80	CONTINUE
C  DO OVER SUBTRACTIONS
	DO 110 IMNS=1,NMNS
	DO 90 I=1,LENGHT
	IF(WRK(I)(:1).EQ.'-')THEN
        IF(I.EQ.LENGHT)STOP'<CALCEXP>:: Error in subtraction 2'
	IF(I.NE.1)READ(WRK(I-1),*)A1
	IF(I.EQ.1)A1=0.D0
	READ(WRK(I+1),*)A2
	AMNS=A1-A2
	IF(I.NE.1)WRITE(WRK(I-1),*)AMNS
	IF(I.EQ.1)WRITE(WRK(I+0),*)AMNS
	IF(I.NE.1)WRK(I+0)=LBLANCK
	WRK(I+1)=LBLANCK
	GOTO100
	ENDIF
  90	CONTINUE
C  REMOVE BLANCKS
 100	CALL REMBL(WRK,LENGHT,NEWL)
	LENGHT=NEWL
 110	CONTINUE
C  DO OVER SUMMATIONS
	DO 140 IPLS=1,NPLS
	DO 120 I=1,LENGHT
	IF(WRK(I)(:1).EQ.'+')THEN
        IF(I.EQ.LENGHT)STOP'<CALCEXP>:: Error in summation 2'
	IF(I.NE.1)READ(WRK(I-1),*)A1
	IF(I.EQ.1)A1=0.D0
	READ(WRK(I+1),*)A2
	APLS=A1+A2
	IF(I.NE.1)WRITE(WRK(I-1),*)APLS
	IF(I.EQ.1)WRITE(WRK(I+0),*)APLS
	IF(I.NE.1)WRK(I+0)=LBLANCK
	WRK(I+1)=LBLANCK
	GOTO130
	ENDIF
 120	CONTINUE
C  REMOVE BLANCKS
 130	CALL REMBL(WRK,LENGHT,NEWL)
	LENGHT=NEWL
 140	CONTINUE
C  CHECK OUT
	IF(LENGHT.NE.1)STOP'<CALCEXP>:: Wrong computation'
C  RETURN
	WORK(NS)=WRK(1)
	END
 
	
	SUBROUTINE SFUNBRA(EXPRESS,J,IEND)
*****************************************************
*  Selects action range for the special function.   *
*****************************************************
	CHARACTER EXPRESS(80)*30
	IF(EXPRESS(J)(:1).NE.'(')
     &	STOP'<CALC>:: Wrong function argument'
	IDOG=0
	DO 10 I=J,80
	IF(EXPRESS(I)(:1).EQ.'(')IDOG=IDOG+1
	IF(EXPRESS(I)(:1).EQ.')')IDOG=IDOG-1
	IF(IDOG.EQ.0)GOTO20
  10	CONTINUE
  20	IEND=I
	END


	SUBROUTINE DFUNBRA(EXPRESS,J,ISTA)
*****************************************************
*  Selects backward action range for the degree.    *
*****************************************************
	CHARACTER EXPRESS(80)*30
	IF(EXPRESS(J)(:1).NE.')')
     &	STOP'<CALC>:: Wrong function argument'
	IDOG=0
	DO 10 I=J,1,-1
	IF(EXPRESS(I)(:1).EQ.')')IDOG=IDOG+1
	IF(EXPRESS(I)(:1).EQ.'(')IDOG=IDOG-1
	IF(IDOG.EQ.0)GOTO20
  10	CONTINUE
  20	ISTA=I
	END




	SUBROUTINE REMBL(EXPRESS,LENGHT,NEWL)
*****************************************************
*  Removes blanck records in the file EXPRESS.      *
*****************************************************
	CHARACTER EXPRESS(80)*30,LBLANCK*30
	LBLANCK='                              '
C  REMOVE BLANCK LINES
	NEWL=LENGHT
	DO 40 I=1,LENGHT
	NBLANCK=0
	DO 10 J=I,NEWL
	IF(EXPRESS(J).EQ.LBLANCK)THEN
	NBLANCK=NBLANCK+1
	ELSE
	GOTO20
	ENDIF
  10	CONTINUE
  20	DO 30 J=I+NBLANCK,NEWL
	EXPRESS(J-NBLANCK)=EXPRESS(J)
  30	CONTINUE
	NEWL=NEWL-NBLANCK
  40	CONTINUE
	DO 50 I=NEWL+1,LENGHT
	EXPRESS(I)=LBLANCK
  50	CONTINUE
	END

	SUBROUTINE CHECKPA(STRING,LENGHT)
*****************************************************
*  Checks for the correct number of paranthesises.  *
*****************************************************
	CHARACTER STRING(80)*1
	ILEFT=0
	IRIGH=0
	DO 10 I=1,LENGHT
	IF(STRING(I).EQ.'(')ILEFT=ILEFT+1
	IF(STRING(I).EQ.')')IRIGH=IRIGH+1
  10	CONTINUE
	IF(ILEFT.NE.IRIGH)PRINT*,'<CALC>:: Paranthesises mismatch'
	IF(ILEFT.NE.IRIGH)STOP
	END


	SUBROUTINE CONVERT(STRING,LENGHT)
*****************************************************
*  Converts upper case letters to lower case.       *
*****************************************************
	CHARACTER STRING(80)*1,LET(26,2)*1
	LET(1 ,1)='A'
	LET(2 ,1)='B'
	LET(3 ,1)='C'
	LET(4 ,1)='D'
	LET(5 ,1)='E'
	LET(6 ,1)='F'
	LET(7 ,1)='G'
	LET(8 ,1)='H'
	LET(9 ,1)='I'
	LET(10,1)='K'
	LET(11,1)='J'
	LET(12,1)='L'
	LET(13,1)='M'
	LET(14,1)='N'
	LET(15,1)='O'
	LET(16,1)='P'
	LET(17,1)='Q'
	LET(18,1)='R'
	LET(19,1)='S'
	LET(20,1)='T'
	LET(21,1)='U'
	LET(22,1)='V'
	LET(23,1)='W'
	LET(24,1)='X'
	LET(25,1)='Y'
	LET(26,1)='Z'
	LET(1 ,2)='a'
	LET(2 ,2)='b'
	LET(3 ,2)='c'
	LET(4 ,2)='d'
	LET(5 ,2)='e'
	LET(6 ,2)='f'
	LET(7 ,2)='g'
	LET(8 ,2)='h'
	LET(9 ,2)='i'
	LET(10,2)='k'
	LET(11,2)='j'
	LET(12,2)='l'
	LET(13,2)='m'
	LET(14,2)='n'
	LET(15,2)='o'
	LET(16,2)='p'
	LET(17,2)='q'
	LET(18,2)='r'
	LET(19,2)='s'
	LET(20,2)='t'
	LET(21,2)='u'
	LET(22,2)='v'
	LET(23,2)='w'
	LET(24,2)='x'
	LET(25,2)='y'
	LET(26,2)='z'
	DO 20 I=1,LENGHT
	DO 10 J=1,26
	IF(STRING(I).EQ.LET(J,1))STRING(I)=LET(J,2)
  10	CONTINUE
  20	CONTINUE 
	END
