	SUBROUTINE GROUPS(A,COORD,IS,LMM,NATOM,NSPIN,NORBS,EPS,IO)
***************************************************************
*   Program finds crystalline group by checking whether       *
*   atoms go to each other after applying group operations    *
*   generated by the WIGMAT subroutine. If spin-orbit         *
*   coupled calculation is necessary, only operations         *
*   which donot change z-axe direction are left together with *
*   inversional operations. Only those leave spin-orbit       *
*   coupling operator invariant.                              *
*     Input:                                                  *
*       A - translational vectors in units of lat.par.        *
*       COORD - positions of atoms in units of lat.par.       *
*       IS(IATOM) - number of type for given atom             *
*       LMM - lmax i.e 0:s,1:p,2:d,3:f,etc                    *
*       NATOM  - number of atoms in primitive cell            *
*       NSPIN  - number of spins                              *
*       NORBS  - 2 if spin and orbit are coupled.             *
*     Output: (in COMMON/GROUPOUT/)                           *
*       NGROUP - number of group elements found               *
*       U - Wigner"s matrices                                 *
*       IP - shows new atom positions after rotation          *
*       SHIFT - shifts if group is non-symmorphic one         *
*       SOPHS - phase shifts of spinor components             *
***************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	INTEGER IS(NPAT),IP(NPAT,NOPRMAX),M3(5),IKOV(NOPRMAX)
     &    ,IKOV0(NOPRMAX),IPT(NPAT)
	DIMENSION A(3,3),COORD(3,NPAT),FAC(0:4*LMAXT),B(3,3)
     &    ,COORD1(3,NPAT),COORD2(3,NPAT),ORIGIN(3),RDCT(3,NPAT)
     &    ,SHIFT(3,NOPRMAX),PSHIFT(3,NPAT),RDC(3,NPAT,NOPRMAX)
     &    ,PR(3,2),PG(3,2),PL(3,2)
	COMPLEX*16 U(NWIGMAX,48),UJ(2,2,48),SOPHS(2,NOPRMAX)
     &    ,YL(-1:1),SOM(2,2),SOU(2,2) 
	COMMON /INVERS/   INV,IKOV,IKOV0,NGROUP0
	COMMON /GROUPOUT/ NGROUP,U,IP,RDC,SHIFT,SOPHS
	COMMON /SPINSOUT/ UJ
	DATA M3/1,2,3,1,2/,PI/3.141592653589793D0/
	CALL FACC(FAC,4*LMAXT)
	DO I=1,3
	B(1,I)=A(1,I)
	DO J=2,3
	B(J,I)=A(J,I)
	ENDDO
	ENDDO
C  A(*,I) IS I-TH TRANSLATION. B'LL BE RECIPROCAL ONES WITHOUT 2PI
	OMEGA=DET(B,3,3)
	DO J=1,3
	DO I=1,3
	B(I,J)=(A(M3(I+1),M3(J+1))*A(M3(I+2),M3(J+2))-
     &  A(M3(I+2),M3(J+1))*A(M3(I+1),M3(J+2)))/OMEGA
	ENDDO
	ENDDO
C  SHIFT THE ORIGIN TO THE FIRST ATOM
	DO I=1,3
	ORIGIN(I)=COORD(I,1)
	ENDDO
	DO IATOM=1,NATOM
	DO I=1,3
	COORD(I,IATOM)=COORD(I,IATOM)-ORIGIN(I)
	ENDDO
	ENDDO
C  REDUCE TO FIRST CELL
	DO IATOM=1,NATOM
	X1=COORD(1,IATOM)+ORIGIN(1)
	Y1=COORD(2,IATOM)+ORIGIN(2)
	Z1=COORD(3,IATOM)+ORIGIN(3)
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	COORD2(1,IATOM)=ABS(P)
	COORD2(2,IATOM)=ABS(Q)
	COORD2(3,IATOM)=ABS(R)
	X1=COORD(1,IATOM)
	Y1=COORD(2,IATOM)
	Z1=COORD(3,IATOM)
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	COORD1(1,IATOM)=ABS(P)
	COORD1(2,IATOM)=ABS(Q)
	COORD1(3,IATOM)=ABS(R)
	IP(IATOM,1)=IATOM
	DO I=1,3
	RDC(I,IATOM,1)=0.0
	ENDDO
	IF(P.LE.1.D-5.AND.Q.LE.1.D-5.AND.R.LE.1.D-5)THEN
	K=0
	DO JATOM=1,NATOM
	IF(IS(JATOM).EQ.IS(IATOM))THEN
	K=K+1
	DO L=1,3
	PSHIFT(L,K)=COORD(L,JATOM)  !!! POSSIBLE SHIFTS
	ENDDO
	ENDIF
	ENDDO
	KS=K
	ENDIF
	ENDDO
C *** CHECK LATTICE FOR THE GROUP OPERATIONS ***
	NG=NGROUP
	IR=0
	DO 2000 IG=1,NG  !!! OVER KOVALEV ROTATIONS
C ROTATE THE ORIGIN
	CALL ROTATE(ORIGIN(1),ORIGIN(2),ORIGIN(3),OX,OY,OZ,U(2,IG),2)
C 1) CHECK LATTICE TRANSLATIONS
	ILAT=0
	DO I=1,3
	CALL ROTATE(A(1,I),A(2,I),A(3,I),X1,Y1,Z1,U(2,IG),2)
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	IF((ABS(P).LE.EPS.OR.ABS(ABS(P)-1.D0).LE.EPS).AND.
     &     (ABS(Q).LE.EPS.OR.ABS(ABS(Q)-1.D0).LE.EPS).AND.
     &     (ABS(R).LE.EPS.OR.ABS(ABS(R)-1.D0).LE.EPS))ILAT=ILAT+1
	ENDDO
	IF(ILAT.LT.3)GOTO2000
C 2) SELECT OPERATIONS WHICH LEAVE SPIN-ORBIT OPERATOR INVARIANT
	IAXE=0
	IF(NORBS.EQ.2)THEN
C Set two arbitrary vectors and their cross-product
	PR(1,1)=0.1D0
	PR(2,1)=0.2D0
	PR(3,1)=0.3D0
	PG(1,1)=0.95D0
	PG(2,1)=0.35D0
	PG(3,1)=0.15D0
	CALL CROSS(PL(1,1),PR(1,1),PG(1,1))
C Apply symmetry operation
	CALL ROTATE(PR(1,1),PR(2,1),PR(3,1),PR(1,2),PR(2,2),PR(3,2)
     &      ,U(2,IG),2)
	CALL ROTATE(PG(1,1),PG(2,1),PG(3,1),PG(1,2),PG(2,2),PG(3,2)
     &      ,U(2,IG),2)
	CALL CROSS(PL(1,2),PR(1,2),PG(1,2))
C PL(*,1..2) original and rotated z angular momenta must be the same
        IF(ABS(PL(3,1)-PL(3,2)).LE.EPS)IAXE=1
C CHECK OUT
	IF(IAXE.EQ.1)THEN
	IERR=0
C Set spin-orbit interaction matrix
        YL(-1)=+0.5D0*SQRT(3.D0/PI/2.D0)*DCMPLX(PL(1,1),-PL(2,1))
        YL( 0)=+0.5D0*SQRT(6.D0/PI/2.D0)*PL(3,1)
        YL(+1)=-0.5D0*SQRT(3.D0/PI/2.D0)*DCMPLX(PL(1,1),+PL(2,1))
	SOM(1,1)=+YL( 0)
	SOM(2,2)=-YL( 0)
	SOM(2,1)=+YL(+1)
	SOM(1,2)=+YL(-1)
C Apply unitary transformation
	DO I=1,2
	DO J=1,2
	SOU(I,J)=(0.D0,0.D0)
	DO M1=1,2
	DO M2=1,2
	SOU(I,J)=SOU(I,J)+DCONJG(UJ(M1,I,IG))*SOM(M1,M2)*UJ(M2,J,IG)
	ENDDO
	ENDDO
	ENDDO
	ENDDO
	IF(ABS(SOU(1,1)-SOM(1,1)).GE.1.D-10)IERR=1
	IF(ABS(SOU(2,2)-SOM(2,2)).GE.1.D-10)IERR=1
C Check whether selected element is represented by diagonal matrix
	IF(ABS(UJ(1,2,IG)).GE.1.D-10.OR.ABS(UJ(2,1,IG)).GE.1.D-10)IERR=2
	IF(IERR.EQ.1)PRINT*,'<GROUPS>:: IERR=1'
	IF(IERR.EQ.2)PRINT*,'<GROUPS>:: IERR=2'
	ENDIF
	ELSEIF(NORBS.EQ.1)THEN
	IAXE=1
	ENDIF
 	IF(IAXE.EQ.0)GOTO2000
C 3) OPERATIONS WITH THE ATOMS IN BASIS
	DO 1000 K=1,KS
	DX=PSHIFT(1,K)
	DY=PSHIFT(2,K)
	DZ=PSHIFT(3,K)
	CALL ROTATE(DX,DY,DZ,X2,Y2,Z2,U(2,IG),1)
	DO IATOM=1,NATOM
	X=COORD(1,IATOM)
	Y=COORD(2,IATOM)
	Z=COORD(3,IATOM)
	CALL ROTATE(X,Y,Z,X1,Y1,Z1,U(2,IG),1)
	X1=X1-X2
	Y1=Y1-Y2
	Z1=Z1-Z2
C  REDUCE TO FIRST CELL
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	CALL CHECK(P,Q,R,COORD1,IPATOM,NATOM,IS,IATOM,EPS)
	IF(IPATOM.EQ.0)GOTO1000
	IPT(IATOM)=IPATOM
	RDCT(1,IATOM)=X1-COORD(1,IPATOM)
	RDCT(2,IATOM)=Y1-COORD(2,IPATOM)
	RDCT(3,IATOM)=Z1-COORD(3,IPATOM)
	ENDDO
C  CHECKING IS DONE, SET UP OPERATION:
	IR=IR+1
	IF(IR.GT.NOPRMAX)STOP'<GROUP>:: NUMBER OF OPERATIONS 
     &  FOUND IS TOO LARGE, INCREASE "NOPRMAX" IN PARAM.DAT'
	DO IATOM=1,NATOM
	IP(IATOM,IR)=IPT(IATOM)
	RDC(1,IATOM,IR)=RDCT(1,IATOM)
	RDC(2,IATOM,IR)=RDCT(2,IATOM)
	RDC(3,IATOM,IR)=RDCT(3,IATOM)
	ENDDO
	IKOV(IR)=IG
	SHIFT(1,IR)=DX-OX+ORIGIN(1)
	SHIFT(2,IR)=DY-OY+ORIGIN(2)
	SHIFT(3,IR)=DZ-OZ+ORIGIN(3)
	IF(NORBS.EQ.1)THEN
	SOPHS(1,IR)=(1.D0,0.D0)
	SOPHS(2,IR)=(1.D0,0.D0)
	ELSEIF(NORBS.EQ.2)THEN
	SOPHS(1,IR)=UJ(1,1,IG)
	SOPHS(2,IR)=UJ(2,2,IG)
	ENDIF
	GOTO2000
1000    CONTINUE
2000    CONTINUE
	NGROUP=IR !! TOTAL # OF GROUP ELEMENTS
	IF(IO.EQ.1)NGROUP0=NGROUP
	DO IR=1,NGROUP
	IF(IO.EQ.1)IKOV0(IR)=IKOV(IR)
	ENDDO
C  SHIFT THE ORIGIN TO THE OLD ONE
	DO IATOM=1,NATOM
	DO I=1,3
	COORD(I,IATOM)=COORD(I,IATOM)+ORIGIN(I)
	ENDDO
	ENDDO
C  CHECK OUT GENERATION
	DO IATOM=1,NATOM
	X=COORD(1,IATOM)
	Y=COORD(2,IATOM)
	Z=COORD(3,IATOM)
	DO IR=1,NGROUP
	IG=IKOV(IR)
	DX=SHIFT(1,IR)
	DY=SHIFT(2,IR)
	DZ=SHIFT(3,IR)
	CALL ROTATE(DX,DY,DZ,X2,Y2,Z2,U(2,IG),1)
	CALL ROTATE(X,Y,Z,X1,Y1,Z1,U(2,IG),1)
	X1=X1-X2
	Y1=Y1-Y2
	Z1=Z1-Z2
C REDUCE TO FIRST CELL
	P=X1*B(1,1)+Y1*B(2,1)+Z1*B(3,1)
	Q=X1*B(1,2)+Y1*B(2,2)+Z1*B(3,2)
	R=X1*B(1,3)+Y1*B(2,3)+Z1*B(3,3)
	P=P-ANINT(P-.4999999D0)
	Q=Q-ANINT(Q-.4999999D0)
	R=R-ANINT(R-.4999999D0)
	CALL CHECK(P,Q,R,COORD2,IPATOM,NATOM,IS,IATOM,EPS)
	IF(IPATOM.EQ.0)STOP'<GROUP>: WRONG GENERATION, IPATOM=0'
	IF(IS(IPATOM).NE.IS(IATOM))STOP'<GROUP>: WRONG GENERATION, IS'
C CHECK RDC
	X2=COORD(1,IPATOM)+RDC(1,IATOM,IR)
	Y2=COORD(2,IPATOM)+RDC(2,IATOM,IR)
	Z2=COORD(3,IPATOM)+RDC(3,IATOM,IR)
	IF(ABS(X2-X1).GT.EPS.OR.ABS(Y2-Y1).GT.EPS.OR.
     &  ABS(Z2-Z1).GT.EPS)STOP'<GROUP>: WRONG GENERATION, RDC'
	ENDDO
	ENDDO
	RETURN
	END



	SUBROUTINE CHECK(P,Q,R,COORD1,IP,NATOM,IS,JATOM,EPS)
***************************************************************
*  Checks if the atoms transform to each other.               *
***************************************************************
	INCLUDE 'PARAM.DAT'
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION COORD1(3,NPAT),IS(NPAT)
	DO IATOM=1,NATOM
	F1=ABS(P-COORD1(1,IATOM))
	F2=ABS(F1-1.D0)
	IF(F1.LE.EPS.OR.F2.LE.EPS)THEN
	F1=ABS(Q-COORD1(2,IATOM))
	F2=ABS(F1-1.D0)
	IF(F1.LE.EPS.OR.F2.LE.EPS)THEN
	F1=ABS(R-COORD1(3,IATOM))
	F2=ABS(F1-1.D0)
	IF(F1.LE.EPS.OR.F2.LE.EPS)THEN
	IP=IATOM
	IF(IS(IP).NE.IS(JATOM))THEN
	IP=0
	ENDIF
	RETURN
	ENDIF
	ENDIF
	ENDIF
	ENDDO
	IP=0
	END



	SUBROUTINE KELITAB
*******************************************************
*  The preparation of Keli table for a given group    *
*******************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	DIMENSION KELTAB(48,48),IKOV(NOPRMAX),RK(3,NOPRMAX)
	COMPLEX*16 U(NWIGMAX,48)
	COMMON /INVERS/   INV,IKOV
	COMMON /GROUPOUT/ NGROUP,U
	COMMON /KELI/     KELTAB
	DATA X/0.1D0/,Y/0.2D0/,Z/0.3D0/
	DO IR=1,NGROUP
	IG=IKOV(IR)
	CALL ROTATE(X,Y,Z,RK(1,IR),RK(2,IR),RK(3,IR),U(2,IG),2)
	ENDDO
	DO IR1=1,NGROUP
	IG1=IKOV(IR1)
	DO IR2=1,NGROUP
	IG2=IKOV(IR2)
	CALL ROTATE(RK(1,IR1),RK(2,IR1),RK(3,IR1),X2,Y2,Z2,U(2,IG2),2)
	DO IR3=1,NGROUP
	IG3=IKOV(IR3)
	IF(ABS(RK(1,IR3)-X2).LE.1.D-05.AND
     &    .ABS(RK(2,IR3)-Y2).LE.1.D-05.AND
     &    .ABS(RK(3,IR3)-Z2).LE.1.D-05)KELTAB(IG2,IG1)=IG3
	ENDDO
	ENDDO
	ENDDO
	END
