	SUBROUTINE WIGMAT(LMM,KOVS,KOVFILE)
***************************************************************
*       Program for preparation of transformation matrices    *
*    for spherical harmonics according to the point group.    *
*    To find spherical harmonic which after applying the      *
*    declared group operation will reduce to Y(M), multiply   *
*    vector Y(M1) to matrix U(M,M1).                          *
*       Program also prepares Wigner"s 2*2 matrices which     *
*    transform spinor components of the wave functions with   *
*    the spin S=1/2                                           *
*       The point group is represented by rotational elements *       
*    which assume either Cubic or Hexagonal symmetry; the     *
*    corresponding operations are listed below in DATACUB and *
*    DATAHEX subroutines. In case, another rotational system  *
*    is required, it must be specified in the file KOVFILE    *
*    according to the rules given by DATACUB,DATAHEX programs.*
*     Input:                                                  *
*       LMM - lmax i.e 0:s,1:p,2:d,3:f,etc                    *
*       KOVS - "C" or "H" system of rotations after Kovalev   *
*       KOVS - "A" together with KOVFILE set up another       *
*                  rotational system                          *
*     Output: (in COMMONS/GROUPOUT/, /SPINSOUT/)              *
*       NGROUP - number of group elements                     *
*       U  - Wigner"s matrices indexed as follows:            *
*           U    (l,g) = U(m'm,l,g) = U(iwig,g)               *
*            m'm                                              *
*  where iwig=l*(2*l-1)*(2*l+1)/3+(2*l+1)*(l+m)+l+m'+1        *
*       UJ - 2*2 Wigner"s matrices of J=1/2                   *
***************************************************************
*  Formulae used after D.A.Varshalovich, A.N. Moskalev,       *
*  V.K. Khersonskii, "Quantum theory of Angular Momentum"     *
*  (World Scientific, Singapure, 1988)                        *
***************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	DIMENSION FAC(0:4*LMAXT),RAXE(3)
	COMPLEX*16 U(NWIGMAX,48),UJ(2,2,48),W
	CHARACTER G*1,KOVS*1,RFILE(0:48,4)*40,STRING(80)*1
     &    ,KOVFILE*60
	COMMON /INVERS/   INV
	COMMON /GROUPOUT/ NGROUP,U
	COMMON /SPINSOUT/ UJ
	DATA PI/3.141592653589793D0/
	PI2=2.D0*PI
	DO I=41,80
	STRING(I)=' '
	ENDDO
	CALL FACC(FAC,4*LMAXT)
C  SELECT ROTATIONAL SYSTEM 
	IF(KOVS.EQ.'C')CALL DATACUB(RFILE)
	IF(KOVS.EQ.'H')CALL DATAHEX(RFILE)
 	IF(KOVS.EQ.'A')CALL DATAARB(RFILE,KOVFILE)
	LM=LMM
C  *** GENERATE WIGNER'S MATRICES ****
	READ(RFILE(0,1),*)NGROUP
	INV=0  !!! inversional ?
	DO 1000 IGG=1,NGROUP
C  READ OPERATION CODE
	READ(RFILE(IGG,1),1010)G
	READ(RFILE(IGG,2),*)IG
	IF(G.EQ.'A')THEN
C  ARBITRARY OPERATION, ROTATION ALONG AXE{X,Y,Z} BY OMEGA
	READ(RFILE(IGG,3),1020)(STRING(I),I=1,40)
        CALL COMPVEC(STRING,ANGLE,1)
	READ(RFILE(IGG,4),1020)(STRING(I),I=1,40)
        CALL COMPVEC(STRING,RAXE,3)
	CALL ROTAXES(RAXE,TETA,FI)
	OM=ANGLE
	V=SIN(OM*0.5D0)*SIN(TETA)
	W=DCMPLX(COS(OM*0.5D0),-SIN(OM*0.5D0)*COS(TETA))
C  SET UP WIGNER'S MATRICES FOR J=1/2
	UJ(1,1,IG)=W
	UJ(2,2,IG)=DCONJG(W)
	UJ(2,1,IG)=(0.D0,-1.D0)*V*EXP((0.D0,+1.D0)*FI)
	UJ(1,2,IG)=(0.D0,-1.D0)*V*EXP((0.D0,-1.D0)*FI)
C  SET UP WIGNER MATRICES OF J=L
        DO L=0,LM
        DO M=-L,L
        DO M1=-L,L
        IWIG=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M1)+L+M+1
        U(IWIG,IG)=(0.D0,0.D0)
        IF(M+M1.GE.0)THEN
        K=0
        DO WHILE(L-M-K.GE.0.AND.L-M1-K.GE.0)
        IF((2*L-M-M1-2*K).EQ.0)THEN
        TM1=1.D0
        ELSE
        TM1=V**(2*L-M-M1-2*K)
        ENDIF
        IF(K.EQ.0)THEN
        TM2=1.D0
        ELSE
        TM2=(V**2-1)**K
        ENDIF
        U(IWIG,IG)=U(IWIG,IG)+SQRT(FAC(L+M)*FAC(L-M)*FAC(L+M1)*
     &  FAC(L-M1))/FAC(K)/FAC(K+M+M1)/FAC(L-M-K)/FAC(L-M1-K)*TM1*TM2
        K=K+1
	ENDDO
        U(IWIG,IG)=U(IWIG,IG)*W**(M+M1)*(0.D0,-1.D0)**(2*L-M-M1)*
     &  EXP(DCMPLX(0.D0,(M1-M)*FI))
        ENDIF
	ENDDO
	ENDDO
	ENDDO
        DO L=0,LM
        DO M=-L,L
        DO M1=-L,L
        IWIG=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M1)+L+M+1
        IWIG1=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L-M1)+L-M+1
        IF(M+M1.LT.0)U(IWIG,IG)=(-1)**(M-M1)*CONJG(U(IWIG1,IG))
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(G.EQ.'Z')THEN
C  ROTATION ALONG Z-AXE
	READ(RFILE(IGG,3),1020)(STRING(I),I=1,40)
        CALL COMPVEC(STRING,ANGLE,1)
	OM=ANGLE
C  SET UP WIGNER'S MATRICES FOR J=1/2
	UJ(1,1,IG)=EXP((0.D0,-1.D0)*OM/2.D0)
	UJ(2,2,IG)=EXP((0.D0,+1.D0)*OM/2.D0)
	UJ(2,1,IG)=(0.D0,0.D0)
	UJ(1,2,IG)=(0.D0,0.D0)
C  SET UP WIGNER MATRICES OF J=L
	DO L=0,LM
	DO M=-L,L
	DO M1=-L,L
	IWIG=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M1)+L+M+1
	IF(M.EQ.M1)THEN
	U(IWIG,IG)=EXP(DCMPLX(0.D0,-M*OM))
	ELSE
	U(IWIG,IG)=(0.D0,0.D0)
	ENDIF
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(G.EQ.'I')THEN
C  INVERSION
	INV=IG
C  SET UP WIGNER'S MATRICES FOR J=1/2
	UJ(1,1,IG)=(0.D0,1.D0)
	UJ(2,2,IG)=(0.D0,1.D0)
	UJ(2,1,IG)=(0.D0,0.D0)
	UJ(1,2,IG)=(0.D0,0.D0)
C  SET UP WIGNER MATRICES OF J=L
	DO L=0,LM
	DO M=-L,L
	DO M1=-L,L
	IWIG=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M1)+L+M+1
	IF(M.EQ.M1)THEN
	U(IWIG,IG)=(-1)**L
	ELSE
	U(IWIG,IG)=(0.D0,0.D0)
	ENDIF
	ENDDO
	ENDDO
	ENDDO
	ELSEIF(G.EQ.'C')THEN
C  COMBINATION OF TWO PREVIOUSLY DEFINED ELEMENTS
	READ(RFILE(IGG,3),*)I1,I2
C  SET UP WIGNER'S MATRICES FOR J=1/2
	CALL PRS(UJ(1,1,I1),UJ(1,1,I2),UJ(1,1,IG))
C  SET UP WIGNER MATRICES OF J=L
	DO L=0,LM
	IWIG=L*(2*L-1)*(2*L+1)/3+1
	CALL PRD(U(IWIG,I1),U(IWIG,I2),U(IWIG,IG),L)
	ENDDO
	ELSEIF(G.EQ.'U')THEN
C  EQUIVALENT OPERATION
C  SET UP WIGNER'S MATRICES FOR J=1/2
	UJ(1,1,IG)=(1.D0,0.D0)
	UJ(2,2,IG)=(1.D0,0.D0)
	UJ(2,1,IG)=(0.D0,0.D0)
	UJ(1,2,IG)=(0.D0,0.D0)
C  SET UP WIGNER MATRICES OF J=L
        DO L=0,LM
        DO M=-L,L
        DO M1=-L,L
        IWIG=L*(2*L-1)*(2*L+1)/3+(2*L+1)*(L+M1)+L+M+1
        IF(M.EQ.M1)THEN
        U(IWIG,IG)=(1.0,0.0)
        ELSE
        U(IWIG,IG)=(0.0,0.0)
        ENDIF
        ENDDO
        ENDDO
        ENDDO
	ENDIF
1000    CONTINUE
	RETURN
1010    FORMAT(A1)
1020    FORMAT(40A1)
	END




	SUBROUTINE ROTAXES(RAXE,TETA,FI)
**********************************************************
*  Determines angles teta and fi from x,y,z.             *
**********************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION RAXE(3)
	DATA PI/3.141592653589793D0/
	X=RAXE(1)/SQRT(RAXE(1)**2+RAXE(2)**2+RAXE(3)**2)
	Y=RAXE(2)/SQRT(RAXE(1)**2+RAXE(2)**2+RAXE(3)**2)
	Z=RAXE(3)/SQRT(RAXE(1)**2+RAXE(2)**2+RAXE(3)**2)
	TETA=ACOS(Z)
C  Check for Z-axe:
	IF(ABS(TETA).LE.1.D-10)THEN
	TETA=1.D-10
	FI=1.D-10
	RETURN
	ENDIF
	IF(ABS(Y).GT.1.D-10)FI=ACOS(X/SIN(TETA))*Y/ABS(Y)
	IF(ABS(Y).LE.1.D-10.AND.X.GE.0.D0)FI=0.D0
	IF(ABS(Y).LE.1.D-10.AND.X.LT.0.D0)FI=PI
	END



	SUBROUTINE PRD(A,B,C,L)
********************************************************
*  Multiplication of Wigner's matrices.                *
********************************************************
	COMPLEX*16 A,B,C
	DIMENSION A(-L:L,-L:L),B(-L:L,-L:L),C(-L:L,-L:L)
	DO M1=-L,L
	DO M2=-L,L
	C(M1,M2)=0.D0
	DO M3=-L,L
	C(M1,M2)=C(M1,M2)+A(M1,M3)*B(M3,M2)
	ENDDO
	ENDDO
	ENDDO
	END




	SUBROUTINE PRS(A,B,C)
********************************************************
*  Multiplication of Wigner's matrices when J=1/2      *
********************************************************
	COMPLEX*16 A,B,C
	DIMENSION A(2,2),B(2,2),C(2,2)
	DO M1=1,2
	DO M2=1,2
	C(M1,M2)=0.D0
	DO M3=1,2
	C(M1,M2)=C(M1,M2)+A(M1,M3)*B(M3,M2)
	ENDDO
	ENDDO
	ENDDO
	END




	SUBROUTINE PRD1(A,B,C)
********************************************************
*  Multiplication of Wigner's matrices.                *
********************************************************
	COMPLEX*16 A,B,C
	DIMENSION A(3),B(3,3),C(3)
	DO I2=1,3
	C(I2)=0.D0
	DO I3=1,3
	C(I2)=C(I2)+A(I3)*B(I3,I2)
	ENDDO
	ENDDO
	END




	SUBROUTINE PRD2(A,B,C)
********************************************************
*  Multiplication of Wigner's matrices.                *
********************************************************
	COMPLEX*16 A,B,C
	DIMENSION A(3),B(3,3),C(3)
	DO I2=1,3
	C(I2)=0.D0
	DO I3=1,3
	C(I2)=C(I2)+A(I3)*CONJG(B(I2,I3)) !!! HERMITIAN CONJUGATED
	ENDDO
	ENDDO
	END




	SUBROUTINE ROTATE(X2,Y2,Z2,X1,Y1,Z1,U,KEY)
*********************************************************
*  Rotates a vector by a group operation.               *
*********************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	COMPLEX*16 U,YG,YG1
	DIMENSION U(3,3),YG(3),YG1(3)
	DATA PI/3.141592653589793D0/
	PI2=2.D0*PI
	R=SQRT(X2*X2+Y2*Y2+Z2*Z2)
	IF(ABS(R).LE.1.D-10)THEN
	X1=0.D0
	Y1=0.D0
	Z1=0.D0
	RETURN
	ENDIF
	X=X2/R
	Y=Y2/R
	Z=Z2/R
	YG(1)=0.5D0*SQRT(3.D0/PI2)*DCMPLX(X,-Y)
	YG(2)=0.5D0*SQRT(6.D0/PI2)*Z
	YG(3)=-0.5D0*SQRT(3.D0/PI2)*DCMPLX(X,Y)
	IF(KEY.EQ.1)CALL PRD1(YG,U,YG1) !!! inversional IG-operation
	IF(KEY.EQ.2)CALL PRD2(YG,U,YG1) !!!      direct IG-operation
	X1=DREAL(YG1(1))*2*SQRT(PI2/3.D0)*R
	Y1=-DIMAG(YG1(1))*2*SQRT(PI2/3.D0)*R
	Z1=YG1(2)*2.D0*SQRT(PI2/6.D0)*R
	RETURN
	END




	SUBROUTINE FACC(F,N)
*******************************************
*  Finds factorial n!                     *
*******************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION F(0:N)
	F(0)=1.D0
	DO I=1,N
	F(I)=F(I-1)*I
	ENDDO 
	END




	SUBROUTINE DATACUB(CKOV)
********************************************************
*  Cubic rotations after Kovalev.                      *
*  Note the difference between the present and Kovalev *
*  numbering which is given by a second column.        *
*  Notations are the following:                        *
*  U - equivalent operation                            *
*  A - arbitrary rotation along (X,Y,Z) (except Z-axe) *
*  Z - rotation along Z                                *
*  I - inversion                                       *
*  C - combination                                     *
********************************************************
	CHARACTER*40 CKOV(0:48,4)
C TOTAL
	CKOV( 0,1)='48                                      '
C EQUIVALENT OPERATION
	CKOV( 1,1)='U                                       '
	CKOV( 1,2)='1                                       '
	CKOV( 1,3)='                                        '
	CKOV( 1,4)='                                        '
C ROTATIONS ALONG X
	CKOV( 2,1)='A                                       '
	CKOV( 2,2)='20                                      '
	CKOV( 2,3)='pi/2                                    '
	CKOV( 2,4)='+1, 0, 0                                '
	CKOV( 3,1)='A                                       '
	CKOV( 3,2)='2                                       '
	CKOV( 3,3)='pi                                      '
	CKOV( 3,4)='+1, 0, 0                                '
	CKOV( 4,1)='A                                       '
	CKOV( 4,2)='19                                      '
	CKOV( 4,3)='3*pi/2                                  '
	CKOV( 4,4)='+1, 0, 0                                '
C ROTATIONS ALONG Y
	CKOV( 5,1)='A                                       '
	CKOV( 5,2)='22                                      '
	CKOV( 5,3)='pi/2                                    '
	CKOV( 5,4)='0,+1, 0                                 '
	CKOV( 6,1)='A                                       '
	CKOV( 6,2)='3                                       '
	CKOV( 6,3)='pi                                      '
	CKOV( 6,4)='0,+1, 0                                 '
	CKOV( 7,1)='A                                       '
	CKOV( 7,2)='24                                      '
	CKOV( 7,3)='3*pi/2                                  '
	CKOV( 7,4)='0,+1, 0                                 '
C ROTATIONS ALONG Z
	CKOV( 8,1)='Z                                       '
	CKOV( 8,2)='15                                      '
	CKOV( 8,3)='pi/2                                    '
	CKOV( 8,4)='0, 0,+1                                 '
	CKOV( 9,1)='Z                                       '
	CKOV( 9,2)='4                                       '
	CKOV( 9,3)='pi                                      '
	CKOV( 9,4)='0, 0,+1                                 '
	CKOV(10,1)='Z                                       '
	CKOV(10,2)='14                                      '
	CKOV(10,3)='3*pi/2                                  '
	CKOV(10,4)='0, 0,+1                                 '
C ROTATIONS ALONG MAIN DIAGONALS
	CKOV(11,1)='A                                       '
	CKOV(11,2)='5                                       '
	CKOV(11,3)='2*pi/3                                  '
	CKOV(11,4)='+1,+1,+1                                '
	CKOV(12,1)='A                                       '
	CKOV(12,2)='9                                       '
	CKOV(12,3)='4*pi/3                                  '
	CKOV(12,4)='+1,+1,+1                                '
	CKOV(13,1)='A                                       '
	CKOV(13,2)='10                                      '
	CKOV(13,3)='2*pi/3                                  '
	CKOV(13,4)='+1,-1,+1                                '
	CKOV(14,1)='A                                       '
	CKOV(14,2)='8                                       '
	CKOV(14,3)='4*pi/3                                  '
	CKOV(14,4)='+1,-1,+1                                '
	CKOV(15,1)='A                                       '
	CKOV(15,2)='6                                       '
	CKOV(15,3)='2*pi/3                                  '
	CKOV(15,4)='-1,-1,+1                                '
	CKOV(16,1)='A                                       '
	CKOV(16,2)='11                                      '
	CKOV(16,3)='4*pi/3                                  '
	CKOV(16,4)='-1,-1,+1                                '
	CKOV(17,1)='A                                       '
	CKOV(17,2)='12                                      '
	CKOV(17,3)='2*pi/3                                  '
	CKOV(17,4)='-1,+1,+1                                '
	CKOV(18,1)='A                                       '
	CKOV(18,2)='7                                       '
	CKOV(18,3)='4*pi/3                                  '
	CKOV(18,4)='-1,+1,+1                                '
C ROTATIONS ALONG OTHER DIAGONALS
	CKOV(19,1)='A                                       '
	CKOV(19,2)='16                                      '
	CKOV(19,3)='pi                                      '
	CKOV(19,4)='+1,+1, 0                                '
	CKOV(20,1)='A                                       '
	CKOV(20,2)='13                                      '
	CKOV(20,3)='pi                                      '
	CKOV(20,4)='+1,-1, 0                                '
	CKOV(21,1)='A                                       '
	CKOV(21,2)='18                                      '
	CKOV(21,3)='pi                                      '
	CKOV(21,4)=' 0,+1,+1                                '
	CKOV(22,1)='A                                       '
	CKOV(22,2)='17                                      '
	CKOV(22,3)='pi                                      '
	CKOV(22,4)=' 0,-1,+1                                '
	CKOV(23,1)='A                                       '
	CKOV(23,2)='23                                      '
	CKOV(23,3)='pi                                      '
	CKOV(23,4)='+1, 0,+1                                '
	CKOV(24,1)='A                                       '
	CKOV(24,2)='21                                      '
	CKOV(24,3)='pi                                      '
	CKOV(24,4)='-1, 0,+1                                '
C INVERSION
	CKOV(25,1)='I                                       '
	CKOV(25,2)='25                                      '
	CKOV(25,3)='                                        '
	CKOV(25,4)='                                        '
C COMBINATIONS
	CKOV(26,1)='C                                       '
	CKOV(26,2)='26                                      '
	CKOV(26,3)='25 2                                    '
	CKOV(26,4)='                                        '
	CKOV(27,1)='C                                       '
	CKOV(27,2)='27                                      '
	CKOV(27,3)='25 3                                    '
	CKOV(27,4)='                                        '
	CKOV(28,1)='C                                       '
	CKOV(28,2)='28                                      '
	CKOV(28,3)='25 4                                    '
	CKOV(28,4)='                                        '
	CKOV(29,1)='C                                       '
	CKOV(29,2)='29                                      '
	CKOV(29,3)='25 5                                    '
	CKOV(29,4)='                                        '
	CKOV(30,1)='C                                       '
	CKOV(30,2)='30                                      '
	CKOV(30,3)='25 6                                    '
	CKOV(30,4)='                                        '
	CKOV(31,1)='C                                       '
	CKOV(31,2)='31                                      '
	CKOV(31,3)='25 7                                    '
	CKOV(31,4)='                                        '
	CKOV(32,1)='C                                       '
	CKOV(32,2)='32                                      '
	CKOV(32,3)='25 8                                    '
	CKOV(32,4)='                                        '
	CKOV(33,1)='C                                       '
	CKOV(33,2)='33                                      '
	CKOV(33,3)='25 9                                    '
	CKOV(33,4)='                                        '
	CKOV(34,1)='C                                       '
	CKOV(34,2)='34                                      '
	CKOV(34,3)='25 10                                   '
	CKOV(34,4)='                                        '
	CKOV(35,1)='C                                       '
	CKOV(35,2)='35                                      '
	CKOV(35,3)='25 11                                   '
	CKOV(35,4)='                                        '
	CKOV(36,1)='C                                       '
	CKOV(36,2)='36                                      '
	CKOV(36,3)='25 12                                   '
	CKOV(36,4)='                                        '
	CKOV(37,1)='C                                       '
	CKOV(37,2)='37                                      '
	CKOV(37,3)='25 13                                   '
	CKOV(37,4)='                                        '
	CKOV(38,1)='C                                       '
	CKOV(38,2)='38                                      '
	CKOV(38,3)='25 14                                   '
	CKOV(38,4)='                                        '
	CKOV(39,1)='C                                       '
	CKOV(39,2)='39                                      '
	CKOV(39,3)='25 15                                   '
	CKOV(39,4)='                                        '
	CKOV(40,1)='C                                       '
	CKOV(40,2)='40                                      '
	CKOV(40,3)='25 16                                   '
	CKOV(40,4)='                                        '
	CKOV(41,1)='C                                       '
	CKOV(41,2)='41                                      '
	CKOV(41,3)='25 17                                   '
	CKOV(41,4)='                                        '
	CKOV(42,1)='C                                       '
	CKOV(42,2)='42                                      '
	CKOV(42,3)='25 18                                   '
	CKOV(42,4)='                                        '
	CKOV(43,1)='C                                       '
	CKOV(43,2)='43                                      '
	CKOV(43,3)='25 19                                   '
	CKOV(43,4)='                                        '
	CKOV(44,1)='C                                       '
	CKOV(44,2)='44                                      '
	CKOV(44,3)='25 20                                   '
	CKOV(44,4)='                                        '
	CKOV(45,1)='C                                       '
	CKOV(45,2)='45                                      '
	CKOV(45,3)='25 21                                   '
	CKOV(45,4)='                                        '
	CKOV(46,1)='C                                       '
	CKOV(46,2)='46                                      '
	CKOV(46,3)='25 22                                   '
	CKOV(46,4)='                                        '
	CKOV(47,1)='C                                       '
	CKOV(47,2)='47                                      '
	CKOV(47,3)='25 23                                   '
	CKOV(47,4)='                                        '
	CKOV(48,1)='C                                       '
	CKOV(48,2)='48                                      '
	CKOV(48,3)='25 24                                   '
	CKOV(48,4)='                                        '
	END 


	SUBROUTINE DATAHEX(HKOV)
********************************************************
*  Hexagonal rotations after Kovalev.                  *
*  Note the difference between the present and Kovalev *
*  numbering which is given by a second column.        *
*  Notations are the following:                        *
*  U - equivalent operation                            *
*  A - arbitrary rotation along (X,Y,Z) (except Z-axe) *
*  Z - rotation along Z                                *
*  I - inversion                                       *
*  C - combination                                     *
********************************************************
	CHARACTER*40 HKOV(0:48,4)
C TOTAL
	HKOV( 0,1)='24                                      '
C EQUIVALENT OPERATION
	HKOV( 1,1)='U                                       '
	HKOV( 1,2)='1                                       '
	HKOV( 1,3)='                                        '
	HKOV( 1,4)='                                        '
C ROTATIONS ALONG Z
	HKOV( 2,1)='Z                                       '
	HKOV( 2,2)='2                                       '
	HKOV( 2,3)='pi/3                                    '
	HKOV( 2,4)='0,0,+1                                  '
	HKOV( 3,1)='Z                                       '
	HKOV( 3,2)='3                                       '
	HKOV( 3,3)='2*pi/3                                  '
	HKOV( 3,4)='0,0,+1                                  '
	HKOV( 4,1)='Z                                       '
	HKOV( 4,2)='4                                       '
	HKOV( 4,3)='pi                                      '
	HKOV( 4,4)='0,0,+1                                  '
	HKOV( 5,1)='Z                                       '
	HKOV( 5,2)='5                                       '
	HKOV( 5,3)='4*pi/3                                  '
	HKOV( 5,4)='0,0,+1                                  '
	HKOV( 6,1)='Z                                       '
	HKOV( 6,2)='6                                       '
	HKOV( 6,3)='5*pi/3                                  '
	HKOV( 6,4)='0,0,+1                                  '
C ROTATIONS IN XY-PLANE
	HKOV( 7,1)='A                                       '
	HKOV( 7,2)='10                                      '
	HKOV( 7,3)='pi                                      '
	HKOV( 7,4)='+1,0,0                                  '
	HKOV( 8,1)='A                                       '
	HKOV( 8,2)='9                                       '
	HKOV( 8,3)='pi                                      '
	HKOV( 8,4)='+sqrt(3)/2,+1/2,0                       '
	HKOV( 9,1)='A                                       '
	HKOV( 9,2)='8                                       '
	HKOV( 9,3)='pi                                      '
	HKOV( 9,4)='+1/2,+sqrt(3)/2,0                       '
	HKOV(10,1)='A                                       '
	HKOV(10,2)='7                                       '
	HKOV(10,3)='pi                                      '
	HKOV(10,4)='0,+1,0                                  '
	HKOV(11,1)='A                                       '
	HKOV(11,2)='12                                      '
	HKOV(11,3)='pi                                      '
	HKOV(11,4)='-1/2,+sqrt(3)/2,0                       '
	HKOV(12,1)='A                                       '
	HKOV(12,2)='11                                      '
	HKOV(12,3)='pi                                      '
	HKOV(12,4)='-sqrt(3)/2,+1/2,0                       '
C INVERSION
	HKOV(13,1)='I                                       '
	HKOV(13,2)='13                                      '
	HKOV(13,3)='                                        '
	HKOV(13,4)='                                        '
C COMBINATIONS
	HKOV(14,1)='C                                       '
	HKOV(14,2)='14                                      '
	HKOV(14,3)='13 2                                    '
	HKOV(14,4)='                                        '
	HKOV(15,1)='C                                       '
	HKOV(15,2)='15                                      '
	HKOV(15,3)='13 3                                    '
	HKOV(15,4)='                                        '
	HKOV(16,1)='C                                       '
	HKOV(16,2)='16                                      '
	HKOV(16,3)='13 4                                    '
	HKOV(16,4)='                                        '
	HKOV(17,1)='C                                       '
	HKOV(17,2)='17                                      '
	HKOV(17,3)='13 5                                    '
	HKOV(17,4)='                                        '
	HKOV(18,1)='C                                       '
	HKOV(18,2)='18                                      '
	HKOV(18,3)='13 6                                    '
	HKOV(18,4)='                                        '
	HKOV(19,1)='C                                       '
	HKOV(19,2)='19                                      '
	HKOV(19,3)='13 7                                    '
	HKOV(19,4)='                                        '
	HKOV(20,1)='C                                       '
	HKOV(20,2)='20                                      '
	HKOV(20,3)='13 8                                    '
	HKOV(20,4)='                                        '
	HKOV(21,1)='C                                       '
	HKOV(21,2)='21                                      '
	HKOV(21,3)='13 9                                    '
	HKOV(21,4)='                                        '
	HKOV(22,1)='C                                       '
	HKOV(22,2)='22                                      '
	HKOV(22,3)='13 10                                   '
	HKOV(22,4)='                                        '
	HKOV(23,1)='C                                       '
	HKOV(23,2)='23                                      '
	HKOV(23,3)='13 11                                   '
	HKOV(23,4)='                                        '
	HKOV(24,1)='C                                       '
	HKOV(24,2)='24                                      '
	HKOV(24,3)='13 12                                   '
	HKOV(24,4)='                                        '
	END


	SUBROUTINE DATAARB(AKOV,KOVFILE)
********************************************************
*  Sets rotational system from KOVFILE.                *
*  Notations are the same as in DATACUB,DATAHEX.       *
********************************************************
	CHARACTER*40 AKOV(0:48,4),KOVFILE*60,G*1
	OPEN(UNIT=1,STATUS='OLD',FILE=KOVFILE)
C READ OPERATIONS FROM KOVFILE
C TOTAL
	READ(1,1010)AKOV(0,1)
	READ(AKOV(0,1),*)NG
	DO IG=1,NG
C OPERATION CODE
	READ(1,1010)AKOV(IG,1)
	READ(AKOV(IG,1),1020)G
C EQUIVALENT
	IF(G.EQ.'U')THEN
	READ(1,1010)AKOV(IG,2)
C ARBITRARY
	ELSEIF(G.EQ.'A')THEN
	READ(1,1010)AKOV(IG,2)
	READ(1,1010)AKOV(IG,3)
	READ(1,1010)AKOV(IG,4)
C Z-AXE
	ELSEIF(G.EQ.'Z')THEN
	READ(1,1010)AKOV(IG,2)
	READ(1,1010)AKOV(IG,3)
C INVERSION
	ELSEIF(G.EQ.'I')THEN
	READ(1,1010)AKOV(IG,2)
C COMBINATION
	ELSEIF(G.EQ.'C')THEN
	READ(1,1010)AKOV(IG,2)
	READ(1,1010)AKOV(IG,3)
	ENDIF
	ENDDO
	CLOSE(1)
1010	FORMAT(A40)
1020	FORMAT(A1)
	END
