      subroutine eigen1a(ndim,h,o,e,nspin,norbs,ierr)
*****************************************************************
*    The solution of the generalized eigenvalue problem         *
*     __                 __                                     *
*     >   H(L"L)*A(L)= E >  O(L"L)A(L)                          *
*     --                 --                                     *
*      L                  L                                     *
*    where H and O matrices are Hermitian and O matrix          *
*    is positive defined.                                       *
*    Taken from standard ASA-TB-LMTO program in Stuttgart.      *
*****************************************************************
      implicit real*8 (a-h,o-z)
      include 'PARAM.DAT'
      parameter (nmax=ndimmax*nspinmax)
      complex*16 h(nmax,nmax),o(nmax,nmax),check
      dimension e(nmax),hr(nmax*nmax*2),or(nmax*nmax*2)
     &  ,z(nmax*nmax*2),wk(nmax*11)
      integer iwk(nmax*11)
      common /store1/hr,or,z,wk,iwk
      data icheck/0/,eps/1.d-07/ ! checking the diagno-program
c  do over spins if no spin-orbit coupling
      ndiag=nspin-norbs+1
      if(mod(ndim,ndiag).ne.0)stop'<eigen1a>: error in dimension'
      mdim=ndim/ndiag
      do 1000 idiag=1,ndiag
      mshf=mdim*(idiag-1)
c  prepare matrices
      do idim=1,mdim
      do jdim=1,mdim
      ind1=idim+mdim*(jdim-1)
      ind2=ind1+mdim*mdim
      hr(ind1)=dreal(h(idim+mshf,jdim+mshf))
      hr(ind2)=dimag(h(idim+mshf,jdim+mshf))
      or(ind1)=dreal(o(idim+mshf,jdim+mshf))
      or(ind2)=dimag(o(idim+mshf,jdim+mshf))
      enddo     
      enddo     
c  call diagonalization program
      call diagno(mdim,hr,or,wk,iwk,z,e(1+mshf),ierr)
      if(icheck.eq.1)then
c  check whether eigenvectors are correct
      do ibnd=1,mdim
      do idim=1,mdim
      check=(0.d0,0.d0)
      do jdim=1,mdim
      ind1=jdim+mdim*(ibnd-1)
      ind2=ind1+mdim*mdim
      check=check+(h(idim+mshf,jdim+mshf)-e(ibnd+mshf)*
     &o(idim+mshf,jdim+mshf))*dcmplx(z(ind1),z(ind2))
      enddo
      if(abs(check).ge.eps)print*,'<EIGEN>: WRONG EIGENVECTORS:',
     &check,idim,ibnd
      enddo
      enddo
c  check whether eigenvectors are orthonormalized
      do ibnd=1,mdim
      do jbnd=1,mdim
      check=(0.d0,0.d0)
      do idim=1,mdim
      ind1=idim+mdim*(ibnd-1)
      ind2=ind1+mdim*mdim
      do jdim=1,mdim
      jnd1=jdim+mdim*(jbnd-1)
      jnd2=jnd1+mdim*mdim
      check=check+dcmplx(z(ind1),-z(ind2))*o(idim+mshf,jdim+mshf)*
     &dcmplx(z(jnd1),z(jnd2))
      enddo
      enddo
      if(ibnd.ne.jbnd.and.abs(check).ge.eps)print*,
     &'<EIGEN>: WRONG ORTHOGONALIZATION:',check,ibnd,jbnd
      if(ibnd.eq.jbnd.and.abs(check-1.d0).ge.eps)print*,
     &'<EIGEN>: WRONG NORMALIZATION:',check,ibnd,jbnd
      enddo
      enddo
      endif
c  save eigenvectors inside the hamiltonian
      do ibnd=1,mdim
      do idim=1,mdim
      ind1=idim+mdim*(ibnd-1)
      ind2=ind1+mdim*mdim
      h(idim+mshf,ibnd+mshf)=dcmplx(z(ind1),z(ind2))
      o(idim+mshf,ibnd+mshf)=dcmplx(z(ind1),z(ind2))
      enddo
      enddo
1000  continue
c  sort energies and eigenvectors
      if(ndiag.eq.2)then
      do ibnd=1,ndim
      wk(ibnd)=e(ibnd)
      enddo
      do ibnds=1,nspin
      do ibndt=1,mdim
      iband=ibnds+nspin*(ibndt-1)
      ipack=ibndt+mdim*(ibnds-1)
      e(iband)=wk(ipack)
      do idim=1,mdim
      imat=idim+mdim*(ibnds-1)
      o(imat,iband)=h(imat,ipack)
      enddo
      enddo
      enddo
      endif
      end




      subroutine diagno(ndim,h,o,wk,iwk,z,eband,ierr)
C- Diagonalizes secular equation with overlap
C----------------------------------------------------------------------
Ci Inputs:
Ci    ndim  :dimension of problem
Cio   h     :hamilton matrix
Cio   o     :overlap matrix
Ci    wk    :work array length at least 11*ndim
Ci    iwk   :work array
Co Outputs:
Cio   h     :hamilton matrix is OVERWRITTEN in this routine
Cio   o     :overlap matrix is OVERWRITTEN in this routine
Co    z     :eigenvectors
Co    eband :eigenvalues
Cr Remarks:
Cr    h,o,z are dimensioned (ndim,ndim,2) (real followed by imag. parts)
C     Find the eigenvalues and vectors of O^-1/2  H  O^-1/2 ---
C----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer ndim,iwk(ndim)
      double precision h(ndim*ndim,2),o(ndim*ndim,2),z(ndim*ndim,2),
     .                 eband(ndim),wk(ndim,11)
C Local variables:
      integer ierr
      character*72 messg1,messg2,messg3,messg4
c     double precision oo(ndim*ndim,2),wkmat(ndim*ndim,2)
c     double precision wkr(ndim),wki(ndim),zoz(ndim*ndim,2)
C External calls:
      external bbak,bchd,binvit,bred,btribk,btridi,dcopy,dinit,
     .         imtqlv
C Data statements:
      data
     .  messg1/' DIAGNO: error in Cholesky decomposition             '/,
     .  messg2/' DIAGNO: error in IMTQLV: trying another subroutine.$'/,
     .  messg3/' DIAGNO: error in BINVIT: trying another subroutine.$'/,
     .  messg4/' DIAGNO: error in IMTQL2.$'/

C--------
c     call dinit(wkmat,2*ndim*ndim)
c     call dinit(zoz,2*ndim*ndim)
c     call dcopy(ndim*ndim*2,o,1,oo,1)
C--------
C --- Cholesky decomposition of overlap matrix
      call bchd (o(1,1),o(1,2),ndim,ierr)
      if (ierr.ne.0) then
        print*,messg1
        call dinit(eband,ndim)
        return
      endif

C --- Reduction of nonorthogonal hermetian matrix to orthogonal form
      call bred(ndim,h(1,1),h(1,2),o(1,1),o(1,2),z(1,1),z(1,2))

C --- Reduction of a complex hermitian matrix to a real symmetric
C --- tridiagonal matrix using unitary similarity transformations
      call btridi(ndim,h(1,1),h(1,2),wk(1,2),wk(1,3),wk(1,4),wk(1,5))

      call imtqlv(ndim,wk(1,2),wk(1,3),wk(1,4),eband,iwk,ierr,wk(1,7))
      if (ierr .ne. 0) then
C	print*,messg2
      else
        call binvit(ndim,wk(1,2),wk(1,3),wk(1,4),ndim,eband,iwk,
     .              z,ierr,wk(1,7),wk(1,8),wk(1,9),wk(1,10),wk(1,11))
C       if (ierr.ne.0)print*,messg3 
      endif
      if (ierr.ne.0) then
C ----- previous method did not work, try this one:
C ----- Make unit eigenvector matrix ---
        call dinit(z,2*ndim*ndim)
        call dcopy(ndim,1.d0,0,z,ndim+1)
        call dcopy(ndim,wk(1,2),1,eband,1)
C ----- Get eigenvalues and eigenvectors of symmetric tridiag. matrix
C ----- by implicit QL method
        call imtql2(ndim,ndim,eband,wk(1,3),z,ierr)
        if (ierr.ne.0)print*,messg4 
      endif

C --- Get eigenvectors of complex hermitian matrix by back transforming
C --- those of the corresponding real symmetric tridiagonal matrix
      call btribk(ndim,h(1,1),h(1,2),wk(1,5),ndim,z(1,1),z(1,2))

C --- Backtransformation of eigenvectors to nonorthogonal representation
C --- Get the eigenvectors of H - E O ---
      call bbak(ndim,o(1,1),o(1,2),z(1,1),z(1,2))

c     call wrmatc(ndim,ndim,z,'z   ')
c     call wrmatc(ndim,ndim,oo,'o   ')
c     call zmult(z,ndim,oo,ndim,wkmat,ndim,.false.)
c     call zmul(wkmat,ndim,z,ndim,zoz,ndim,wkr,wki,.false.)
c     call wrmatc(ndim,ndim,zoz,'zoz ')
      end


      subroutine bbak(n,or,oi,zr,zi)
C- Back-transforms eigenvectors to nonorthogonal representation
C ----------------------------------------------------------------------
Ci Inputs:
Ci   n     :order of the matrix system
Ci   o     :Cholesky-decomposed overlap matrix
Cio  z     :eigenvectors to be back transformed
Co Outputs:
Cio  z     :transformed eigenvectors
Cr Remarks:
Cr   Adapted from rebakh, eispack
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer n
      double precision or(*),oi(*),zr(*),zi(*)
C Local variables:
      integer i,j,k,im1n,jm1n,ij,ii,ki,kj
      double precision xr,xi

      do j = 1, n
         jm1n=(j-1)*n
         do i = n, 1, -1
            im1n=(i-1)*n
            ij=i+jm1n
            ii=i+im1n
            xr = zr(ij)
            xi = zi(ij)
            do k = i+1, n
              ki=k+im1n
              kj=k+jm1n
              xr = xr - or(ki)*zr(kj) - oi(ki)*zi(kj)
              xi = xi - or(ki)*zi(kj) + oi(ki)*zr(kj)
            enddo
            zr(ij) = xr/or(ii)
            zi(ij) = xi/or(ii)
         enddo
      enddo
      end


      subroutine bchd(or,oi,n,ierr)
C- Cholesky decomposition of hermetian matrix
C ----------------------------------------------------------------------
Ci Inputs:
Ci   n     :order of the matrix system
Cio  o     :overlap matrix (only full upper triangle is used)
Co Outputs:
Cio  o     :full lower triangle of Cholesky factor l of o
Co          The strict upper triangle is the cc of the lower one
Co   ierr  :nonzero if decomposition finds matrix not pos. def.
Cr Remarks:
Cr   Adapted from hchd which is adapted from eispack reduc for
Cr   hermetian matrices. bchd is written for square matrices.
Cr   At the end the strict lower triangle is set to the
Cr   complex conjugated of the strict upper one. Then it is possible to
Cr   change the indices in particular for the innermost loop so
Cr   that the first array index equals the loop variable in bbak
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer ierr,n
      double precision or(n*n),oi(n*n)
C Local variables:
      integer i,j,k,im1,im1n,jm1n,ij,ji,ki,kj
      double precision xr,xi,y

      do j = 1, n
        jm1n=(j-1)*n
        do i = 1, j
          im1n=(i-1)*n
          ij=i+jm1n
          ji=j+im1n
          or(ji) = or(ij)
          oi(ji) = oi(ij)
        enddo
      enddo

C --- form l in the arrays o
      do i = 1, n
         im1 = i - 1
         im1n=im1*n
         do j = i, n
            jm1n=(j-1)*n
            ij=i+jm1n
            ji=j+im1n
            xr =  or(ji)
            xi = -oi(ji)
            do k = 1, im1
              ki=k+im1n
              kj=k+jm1n
c             xr = xr - or(ki)*or(kj) - oi(ki)*oi(kj)
c             xi = xi - or(ki)*oi(kj) + oi(ki)*or(kj)
              xr = xr - or(ki)*or(kj) - oi(ki)*oi(kj)
              xi = xi + or(ki)*oi(kj) - oi(ki)*or(kj)
            enddo
            if (j.ne.i) then
              or(ji) = xr / y
              oi(ji) = xi / y
              or(ij) =  or(ji)
              oi(ij) = -oi(ji)
c             oi(ij) =  oi(ji)
            else
              ierr = i
              if (xr .le. 0.0d0) return
              y = dsqrt(xr)
              or(ji) = y
              oi(ji) = 0.d0
            endif
         enddo
      enddo
      ierr = 0
      end



      subroutine binvit(n,d,e,e2,m,w,ind,z,ierr,rv1,rv2,rv3,rv4,rv6)
C- Finds eigenvectors of a tridiagonal symmetric matrix
C ----------------------------------------------------------------------
Ci Inputs:
Ci   n     :order of input matrix
Ci   d     :diagonal elements of input matrix
Ci   e     :subdiagonal elements of input matrix in position 2,...,n-1
Ci   e2    :square of e
Ci   m     :number of specified eigenvalues
Ci   w     :eigenvalues
Ci   ind   :index to submatrix
Cw Work arrays:
Cw   rv1   :work array
Cw   rv2   :work array
Cw   rv3   :work array
Cw   rv4   :work array
Cw   rv6   :work array
Co Outputs:
Co   z     :orthonormal eigenvector
Co   ierr  :0 for normal return
Cr Remarks:
Cr    AUTHORS -
Cr       THIS ROUTINE IS A MODIFICATION OF EISPACK ROUTINE TINVIT
Cr       WHICH IS A TRANSLATION OF THE INVERSE ITERATION TECHNIQUE
Cr       IN THE ALGOL PROCEDURE TRISTURM BY PETERS AND WILKINSON.
Cr       HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 418-439(1971).
Cr       THE EISPACK VERSION WAS MODIFIED BY C. MOLER AND D. SPANGLER
Cr       (NRCC - 1 APR 1979).  THIS VERSION IS BY S. T. ELBERT
Cr       (AMES LABORATORY - USDOE), WHO FURTHER STREAMLINED THE CODE.
Cr
Cr    PURPOSE -
Cr       THIS ROUTINE FINDS THOSE EIGENVECTORS OF A TRIDIAGONAL
Cr       SYMMETRIC MATRIX CORRESPONDING TO SPECIFIED EIGENVALUES,
Cr       USING INVERSE ITERATION.
Cr
Cr    METHOD -
Cr       THE CALCULATIONS PROCEED AS FOLLOWS.  FIRST, THE  E2  ARRAY
Cr       IS INSPECTED FOR THE PRESENCE OF A ZERO ELEMENT DEFINING A
Cr       SUBMATRIX.  THE EIGENVALUES BELONGING TO THIS SUBMATRIX ARE
Cr       IDENTIFIED BY THEIR COMMON SUBMATRIX INDEX IN  IND.
Cr
Cr       THE EIGENVECTORS OF THE SUBMATRIX ARE THEN COMPUTED BY
Cr       INVERSE ITERATION.  FIRST, THE  LU  DECOMPOSITION OF THE SUB-
Cr       MATRIX WITH AN APPROXIMATE EIGENVALUE SUBTRACTED FROM ITS
Cr       DIAGONAL ELEMENTS IS ACHIEVED BY GAUSSIAN ELIMINATION USING
Cr       PARTIAL PIVOTING.  THE MULTIPLIERS DEFINING THE LOWER
Cr       TRIANGULAR MATRIX  L  ARE STORED IN THE TEMPORARY ARRAY  RV4
Cr       AND THE UPPER TRIANGULAR MATRIX  U  IS STORED IN THE THREE
Cr       TEMPORARY ARRAYS  RV1,  RV2, AND  RV3.  SAVING THESE
Cr       QUANTITIES IN RV1,  RV2,  RV3, AND  RV4 AVOIDS REPEATING
Cr       THE  LU  DECOMPOSITION IF FURTHER ITERATIONS ARE REQUIRED.
Cr       AN APPROXIMATE VECTOR, STORED IN  RV6, IS COMPUTED STARTING
Cr       FROM AN INITIAL VECTOR, AND THE NORM OF THE APPROXIMATE
Cr       VECTOR IS COMPARED WITH A NORM OF THE SUBMATRIX TO DETERMINE
Cr       WHETHER THE GROWTH IS SUFFICIENT TO ACCEPT IT AS AN
Cr       EIGENVECTOR.  IF THIS VECTOR IS ACCEPTED, ITS EUCLIDIAN NORM
Cr       IS MADE 1.  IF THE GROWTH IS NOT SUFFICIENT, THIS VECTOR IS
Cr       USED AS THE INITIAL VECTOR IN COMPUTING THE NEXT APPROXIMATE
Cr       VECTOR.  THIS ITERATION PROCESS IS REPEATED AT MOST  5  TIMES.
Cr
Cr       EIGENVECTORS COMPUTED IN THE ABOVE WAY CORRESPONDING TO WELL-
Cr       SEPARATED EIGENVALUES OF THIS SUBMATRIX WILL BE ORTHOGONAL.
Cr       HOWEVER, EIGENVECTORS CORRESPONDING TO CLOSE EIGENVALUES OF
Cr       THIS SUBMATRIX MAY NOT BE SATISFACTORILY ORTHOGONAL.  HENCE,
Cr       TO INSURE ORTHOGONAL EIGENVECTORS, EACH APPROXIMATE VECTOR IS
Cr       MADE ORTHOGONAL TO THOSE PREVIOUSLY COMPUTED EIGENVECTORS
Cr       WHOSE EIGENVALUES ARE CLOSE TO THE CURRENT EIGENVALUE, AS
Cr       DETERMINED BY THE PARAMETER GRPTOL.
Cr       IF THE ORTHOGONALIZATION PROCESS PRODUCES A ZERO VECTOR, A
Cr       COLUMN OF THE IDENTITY MATRIX IS USED AS AN INITIAL VECTOR
Cr       FOR THE NEXT ITERATION.
Cr
Cr       IDENTICAL EIGENVALUES ARE PERTURBED SLIGHTLY IN AN ATTEMPT TO
Cr       OBTAIN INDEPENDENT EIGENVECTORS.  THESE PERTURBATIONS ARE NOT
Cr       RECORDED IN THE EIGENVALUE ARRAY  W.
Cr
Cr       THE ABOVE STEPS ARE REPEATED ON EACH SUBMATRIX UNTIL ALL THE
Cr       EIGENVECTORS ARE COMPUTED.
Cr
Cr    ON ENTRY-
Cr
Cr       N      - INTEGER
Cr                THE ORDER OF THE MATRIX.
Cr
Cr       D      - W.P. REAL (N)
Cr                THE DIAGONAL ELEMENTS OF THE INPUT MATRIX.
Cr
Cr       E      - W.P. REAL (N)
Cr                CONTAINS THE SUBDIAGONAL ELEMENTS OF THE INPUT MATRIX
Cr                IN ITS LAST N-1 POSITIONS.  E(1) IS ARBITRARY.
Cr
Cr       E2     - W.P. REAL (N)
Cr          CONTAINS THE SQUARES OF THE CORRESPONDING ELEMENTS OF E,
Cr          WITH ZEROS CORRESPONDING TO NEGLIGIBLE ELEMENTS OF E.
Cr          E(I) IS CONSIDERED NEGLIGIBLE IF IT IS NOT LARGER THAN
Cr          THE PRODUCT OF THE RELATIVE MACHINE PRECISION AND THE SUM
Cr          OF THE MAGNITUDES OF D(I) AND D(I-1).  E2(1) MUST CONTAIN
Cr          0.0 IF THE EIGENVALUES ARE IN ASCENDING ORDER, OR 2.0
Cr          IF THE EIGENVALUES ARE IN DESCENDING ORDER.  IF  BISECT,
Cr          TRIDIB, OR  EIMQLV  HAS BEEN USED TO FIND THE EIGENVALUES,
Cr          THEIR OUTPUT E2 ARRAY IS EXACTLY WHAT IS EXPECTED HERE.
Cr
Cr       M      - INTEGER
Cr                THE NUMBER OF SPECIFIED EIGENVALUES.
Cr
Cr       W      - W.P. REAL (M)
Cr                THE M EIGENVALUES IN ASCENDING OR DESCENDING ORDER.
Cr
Cr       IND    - INTEGER (M)
Cr           CONTAINS IN ITS FIRST M POSITIONS THE SUBMATRIX INDICES
Cr           ASSOCIATED WITH THE CORRESPONDING EIGENVALUES IN W --
Cr           1 FOR EIGENVALUES BELONGING TO THE FIRST SUBMATRIX FROM
Cr           TOP, 2 FOR THOSE BELONGING TO THE SECOND SUBMATRIX, ETC.
Cr
Cr    ON EXIT-
Cr
Cr       Z      - W.P. REAL (NM,M)
Cr          CONTAINS THE ASSOCIATED SET OF ORTHONORMAL EIGENVECTORS.
Cr          ANY VECTOR WHICH FAILS TO CONVERGE IS SET TO ZERO.
Cr
Cr       IERR   - INTEGER
Cr                SET TO
Cr                ZERO   FOR NORMAL RETURN,
Cr                -R     IF THE EIGENVECTOR CORRESPONDING TO THE R-TH
Cr                       EIGENVALUE FAILS TO CONVERGE IN 5 ITERATIONS.
Cr
Cr       RV1, RV2, RV3, RV4, AND RV6  -  W.P. REAL (N)
Cr                TEMPORARY STORAGE ARRAYS.
Cr
Cr    COMPLEXITY -
Cr       O(N*M)
Cr
Cr    NOTE -
Cr    QUESTIONS AND COMMENTS ABOUT EISPACK SHOULD BE DIRECTED TO
Cr    B. S. GARBOW, APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LAB.
Cr
C-----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer ind(*),ierr,n,m
      double precision d(n),e(n),e2(n),w(m),z(n,m),
     .                 rv1(n),rv2(n),rv3(n),rv4(n),rv6(n)
C Local variables:
      integer i,igroup,ip,ipm1,ipp1,iq,iqm1,iqm2,iqp1,ir,is,itag,iters,
     .        j,jj,nsub,nsubup
      double precision anorm,anorm2,epmach,eps2,eps3,eps4,order,
     .                 test,u,uk,v,xu,rtpre,rtcur,abs,parm1,parm2

      ierr = 0
      if (m .le. 0) return
C --- EPMACH IS A MACHINE DEPENDENT PARAMETER SPECIFYING
C --- THE RELATIVE PRECISION OF FLOATING POINT ARITHMETIC.
      epmach = 0.5d0**20
      do i=1,100
        epmach = epmach*0.5d0
        test = 1.d0 + epmach
        if(test .eq. 1.d0) goto 120
      enddo
  120 continue
      epmach=epmach+epmach
      epmach=epmach+epmach

      itag = 0
      order = 1.d0 - e2(1)
      ip = 1
      do iq=1,n
       if (iq.eq.n .or. e2(iq+1) .eq. 0.d0) then
C ----- ESTABLISH AND PROCESS NEXT SUBMATRIX WITH INDEX RANGE IP TO IQ
        itag = itag + 1
        is = 0
        rtcur=w(1)
        do ir = 1, m
          if (ind(ir) .eq. itag) then
            rtpre = rtcur
            rtcur = w(ir)
            ipp1 = ip + 1
            ipm1 = ip - 1
            nsub = iq - ipm1
            if (is .eq. 0) then
C ----------- CHECK FOR ISOLATED ROOT
              xu = 1.d0
              if (ip .eq. iq) then
C ------------- SET ISOLATED ROOT TO UNIT VECTOR
                do i=1,n
                  z(i,ir)=0.d0
                enddo
                z(ip,ir)=1.d0
                goto 920
              endif
              anorm = dabs(d(ip))
              do i = ipp1, iq
                anorm = anorm + dabs(d(i)) + dabs(e(i))
              enddo
C ----------- EPS2 IS THE CRITERION FOR GROUPING,
C ----------- EPS3 REPLACES ZERO PIVOTS AND EQUAL ROOTS ARE
C ----------- MODIFIED BY EPS3
C ----------- EPS4 IS TAKEN VERY SMALL TO AVOID OVERFLOW
              uk = dble(nsub)
              anorm2 = anorm / uk
              eps2 = 1.d-5 * anorm2
              eps3 = epmach * anorm
              eps4 = uk * eps3
              uk = eps4 / dsqrt(uk)
              is = ip
              igroup = 0
C ----------- LOOK FOR CLOSE OR COINCIDENT ROOTS
            else
              igroup = igroup + 1
              if (dabs(rtcur-rtpre) .ge. eps2) igroup =0
              if (order * (rtcur - rtpre) .le. 0.d0)
     .           rtcur = rtpre + order * eps3
C ----------- ELIMINATION WITH INTERCHANGES AND INITIALIZATION OF
C ----------- VECTOR
            endif
            v = 0.d0
            iqm1 = iq - 1
            rv6(ip) = uk
            u = d(ip) - rtcur
            if(ip .ne. iq) then
              v = e(ipp1)
              if (ipp1 .le. iqm1) then
                do i = ipp1,iqm1
                  rv6(i) = uk
                  if (dabs(e(i)) .ge. dabs(u)) then
C ----------------- WARNING -- A DIVIDE CHECK MAY OCCUR HERE
C ----------------- IF E2 ARRAY HAS NOT BEEN SPECIFIED CORRECTLY
                    xu = u / e(i)
                    rv4(i) = xu
                    rv1(i-1) = e(i)
                    rv2(i-1) = d(i) - rtcur
                    rv3(i-1) = e(i+1)
                    u = v - xu * rv2(i-1)
                    v = -xu * rv3(i-1)
                  else
                    xu = e(i) / u
                    rv4(i) = xu
                    rv1(i-1) = u
                    rv2(i-1) = v
                    rv3(i-1) = 0.d0
                    u = d(i) - rtcur - xu * v
                    v = e(i+1)
                  endif
                enddo
              endif
              rv6(iq) = uk
              rv3(iqm1) = 0.d0
              if (dabs(e(iq)) .ge. dabs(u)) then
                xu = u / e(iq)
                rv4(iq) = xu
                rv1(iqm1) = e(iq)
                rv2(iqm1) = d(iq) - rtcur
                u = v - xu * rv2(iqm1)
                v = 0.d0
              else
                xu = e(iq) / u
                rv4(iq) = xu
                rv1(iqm1) = u
                rv2(iqm1) = v
                u = d(iq) - rtcur - xu * v
              endif
            endif
            if (dabs(u) .lt. eps3) u = eps3
            rv1(iq) = u
            rv2(iq) = 0.d0
            rv3(iq) = 0.d0
C --------- BACK SUBSTITUTION FOR I=IQ STEP -1 UNTIL IP DO
            do iters=1,5
              rv6(iq) = rv6(iq)/rv1(iq)
              u = rv6(iq)
              rv6(iqm1) = (rv6(iqm1) - u * rv2(iqm1)) / rv1(iqm1)
              v = u
              u = rv6(iqm1)
              anorm = dabs(u) + dabs(v)
              if (ipp1 .le. iqm1) then
                iqm2=iqm1-1
                do i = iqm2,ip,-1
                  rv6(i) = (rv6(i) - u*rv2(i) - v*rv3(i)) / rv1(i)
                  v = u
                  u = rv6(i)
                  anorm = anorm + dabs(u)
                enddo
              endif
              if (igroup .ne. 0) then
C ------------- ORTHOGONALIZE WITH RESPECT TO PREVIOUS MEMBERS OF GROUP
                j = ir
                do jj = 1,igroup
  630             j = j - 1
                  if (ind(j) .ne. itag) goto 630
                  xu = 0d0
                  nsubup = nsub+ip-1
                  do i = ip, nsubup
                    xu = xu + rv6(i)*z(i,j)
                  enddo
                  xu = -xu
                  if (xu .ne. 0d0) then
                    do i = ip, nsubup
                      rv6(i) = rv6(i) + xu*z(i,j)
                    enddo
                  endif
                enddo
                anorm = 0.d0
                do i = ip,iq
                  anorm = anorm + dabs(rv6(i))
                enddo
              endif
              if (anorm .lt. 1.d0) then
C ------------- FORWARD SUBSTITUTION
                if (anorm .eq. 0.d0) then
                  rv6(is) = eps4
                  is = is + 1
                  if (is .gt. iq) is = ip
                else
                  xu = eps4 / anorm
                  do i =ip,iq
                    rv6(i) = rv6(i) * xu
                  enddo
                endif
C ------------- ELIMINATION OPERATIONS ON NEXT VECTOR ITERATE
                do i = ipp1,iq
                  u = rv6(i)
C --------------- IF RV1(I-1) .EQ. E(I), A ROW INTERCHANGE
C --------------- WAS PERFORMED EARLIER IN THE
C --------------- TRIANGULARIZATION PROCESS
                  if (rv1(i-1) .eq. e(i)) then
                    u = rv6(i-1)
                    rv6(i-1) = rv6(i)
                  endif
                  rv6(i) = u - rv4(i) * rv6(i-1)
                enddo
              else
                goto 840
              endif
            enddo
C --------- SET ERROR -- NON-CONVERGED EIGENVECTOR
            ierr = -ir
            do i=1,n
               z(i,ir)=0.d0
            enddo
            goto 920
C --------NORMALIZE SO THAT SUM OF SQUARES IS 1 AND EXPAND TO FULL ORDER
840         u = 0.d0
            do i = ip,iq
              u = u + rv6(i)*rv6(i)
            enddo
            xu = 1.d0 / dsqrt(u)
            if(ipm1.ge.1) then
              do i = 1, ipm1
                z(i,ir) = 0.d0
              enddo
            endif
            do i = ip,iq
              z(i,ir) = rv6(i) * xu
            enddo
            iqp1 = iq + 1
            if(iqp1 .le. n) then
              do i=iqp1,n
                z(i,ir) = 0.d0
              enddo
            endif
          endif
  920     continue
        enddo
        ip = iq + 1
       endif
      enddo
      return
      end



      subroutine bred(n,hr,hi,or,oi,htr,hti)
C- Reduces nonorthogonal hermetian matrix to orthogonal form
C ----------------------------------------------------------------------
Ci Inputs:
Ci   n     :order of the matrix system.
Cio  h     :hamiltonian matrix(only full upper triangle is used)
Ci   o     :Cholesky-decomposed overlap matrix
Cw Work arrays
Cw   ht    :work array
Co Outputs:
Cio  h     :reduced hermetian matrix:  a <-  ldag^-1 a l^-1
Co          The strict upper triangle
Cr Remarks:
Cr   Adapted from hred which is adapted from eispack reduc for
Cr   hermetian matrices. bred is written for square matrices.
Cr   At the beginning the strict upper triangle of ht is set to the
Cr   complex conjugated of the strict lower one of h.
Cr   Then it is possible to
Cr   change the indices in particular for the innermost loop so
Cr   that the first array index equals the loop variable.
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer n
      double precision hr(n*n),hi(n*n),or(n*n),oi(n*n),
     .                 htr(n*n),hti(n*n)
C Local variables:
      integer i,ii,im1,j,jm1,k,im1n,jm1n,ij,ji,ki,kj
      double precision xr,xi,y

      do j = 1, n
        jm1n=(j-1)*n
        do i = 1, j
          ij=i+jm1n
          htr(ij) =  hr(ij)
          hti(ij) = -hi(ij)
        enddo
      enddo

C --- form the transpose of the upper triangle of inv(l)*h
C --- in the lower triangle of the array h
      do i = 1, n
         im1 = i - 1
         im1n=im1*n
         ii=i+im1n
         y = or(ii)

Ci ----  (l^-1 * h)_ij  =  (h_ij - sum_k<i  l_ik * (l^-1)_kj) / l_ii
         do j = i, n
            jm1n=(j-1)*n
            ij=i+jm1n
            ji=j+im1n
            xr = hr(ij)
            xi = hi(ij)
            do k = 1, im1
              ki=k+im1n
              kj=k+jm1n
c             xr = xr - or(ki)*htr(kj) + oi(ki)*hti(kj)
c             xi = xi - or(ki)*hti(kj) - oi(ki)*htr(kj)
              xr = xr - or(ki)*htr(kj) - oi(ki)*hti(kj)
              xi = xi - or(ki)*hti(kj) + oi(ki)*htr(kj)
            enddo
            hr(ji) = xr / y
            hi(ji) = xi / y
            htr(ij) = hr(ji)
            hti(ij) = hi(ji)
         enddo
      enddo
C --- pre-multiply by inv(l) and overwrite
      do j = 1, n
         jm1 = j - 1
         jm1n=jm1*n
         do i = j, n
            im1 = i - 1
            im1n=im1*n
            ii=i+im1n
            ij=i+jm1n
            ji=j+im1n
            xr =  hr(ij)
            xi = -hi(ij)
            do k = j, im1
              ki=k+im1n
              kj=k+jm1n
c             xr = xr - htr(kj)*or(ki) + hti(kj)*oi(ki)
c             xi = xi - htr(kj)*oi(ki) - hti(kj)*or(ki)
              xr = xr - htr(kj)*or(ki) - hti(kj)*oi(ki)
              xi = xi + htr(kj)*oi(ki) - hti(kj)*or(ki)
            enddo
            do k = 1, jm1
              ki=k+im1n
              kj=k+jm1n
c             xr = xr - hr(kj)*or(ki) - hi(kj)*oi(ki)
c             xi = xi - hr(kj)*oi(ki) + hi(kj)*or(ki)
              xr = xr - hr(kj)*or(ki) + hi(kj)*oi(ki)
              xi = xi + hr(kj)*oi(ki) + hi(kj)*or(ki)
            enddo
            hr(ji) = xr / or(ii)
            hi(ji) = xi / or(ii)
            htr(ij) = hr(ji)
            hti(ij) = hi(ji)
         enddo
      enddo
      end



      subroutine btribk(n,ar,ai,tau,m,zr,zi)
C- Get eigenvectors of a complex hermitian matrix by back transforming
C- those of the corresponding real symmetric tridiagonal matrix
C ----------------------------------------------------------------------
Ci Inputs:
Ci   n     :order of the matrix
Ci   ar    :contains information about the unitary transformations used
Ci          in the reduction by btridi (in strict upper triangle)
Ci   ai    :contains information about the unitary transformations used
Ci          in the reduction by btridi (in full upper triangle)
Ci   tau   :contains further information about the transformations
Ci         (output from btridi)
Ci   m     :number of eigenvectors to be back transformed
Ci   zr    :contains the eigenvectors to be back transformed
Co Outputs:
Co   zr    :contain the real parts of the transformed eigenvectors
Co   zi    :contain the imaginary parts of the transformed eigenvectors
Cr Remarks:
Cr    this subroutine is a translation of a complex analogue of
Cr    the algol procedure trbak1, num. math. 11, 181-195(1968)
Cr    by martin, reinsch, and wilkinson.
Cr    handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
Cr
Cr    this subroutine forms the eigenvectors of a complex hermitian
Cr    matrix by back transforming those of the corresponding
Cr    real symmetric tridiagonal matrix determined by  btridi.
Cr
Cr    note that the last component of each returned vector
Cr    is real and that vector euclidean norms are preserved.
Cr
Cr    questions and comments should be directed to burton s. garbow,
Cr    mathematics and computer science div, argonne national laboratory
Cr
Cr    this version is adapted from the eispack htridi. In contrast
Cr    only the upper triangle of a is used for input
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer m,n
      double precision ar(n*n),ai(n*n),tau(2,n),zr(n*m),zi(n*m)
C Local variables:
      integer i,j,jm1,k,l,im1n,jm1n,ki,kj
      double precision h,s,si,smin,ddot
C
      if (m .eq. 0) return
C --- transform the eigenvectors of the real symmetric
C --- tridiagonal matrix to those of the hermitian tridiagonal matrix.

      do j = 1, m
        jm1=j-1
        do k = 1, n
          kj=k+jm1*n
          zi(kj) = -zr(kj) * tau(2,k)
          zr(kj) =  zr(kj) * tau(1,k)
        enddo
      enddo

      if (n .eq. 1) return
C --- recover and apply the householder matrices
      do i = 2, n
        im1n=(i-1)*n
        l = i - 1
        h = ai(i+im1n)
        if (h .ne. 0.0d0) then
          do j = 1, m
            jm1n=(j-1)*n
c#ifdefc BLAS
c             s  = ddot(l,ar(im1n+1),1,zr(jm1n+1),1)
c     .          - ddot(l,ai(im1n+1),1,zi(jm1n+1),1)
c             si = ddot(l,ar(im1n+1),1,zi(jm1n+1),1)
c     .          + ddot(l,ai(im1n+1),1,zr(jm1n+1),1)
c#else
            s  = 0.0d0
            si = 0.0d0
            do k = 1, l
              ki=k+im1n
              kj=k+jm1n
              s =  s  + ar(ki) * zr(kj) - ai(ki) * zi(kj)
              si = si + ar(ki) * zi(kj) + ai(ki) * zr(kj)
            enddo
c#endif

C --------- double divisions avoid possible underflow
            s = (s / h) / h
            smin = -s
            si = - (si / h) / h

c#ifdefc BLAS
c            call daxpy(l,smin,ar(im1n+1),1,zr(jm1n+1),1)
c            call daxpy(l,si  ,ai(im1n+1),1,zr(jm1n+1),1)
c            call daxpy(l,si  ,ar(im1n+1),1,zi(jm1n+1),1)
c            call daxpy(l,s   ,ai(im1n+1),1,zi(jm1n+1),1)
c#else
            do k = 1, l
              ki=k+im1n
              kj=k+jm1n
              zr(kj) = zr(kj) + smin * ar(ki) + si * ai(ki)
              zi(kj) = zi(kj) + si   * ar(ki) + s  * ai(ki)
            enddo
c#endif
          enddo
        endif
      enddo
      end




      subroutine btridi(n,ar,ai,d,e,e2,tau)
C- Reduces a complex hermitian matrix to a real symmetric tridiagonal
C- matrix using unitary similarity transformations
C ----------------------------------------------------------------------
Ci Inputs:
Ci   n     :order of matrix
Cio  ar    :real part of the complex hermitian input matrix
Cio         only the upper triangle of the matrix need be supplied
Cio  ai    :imaginary part of the complex hermitian input matrix
Cio         only the upper triangle of the matrix need be supplied
Co Outputs:
Cio  ar    :contains information about the unitary transformations used
Cio         in the reduction (in strict upper triangle)
Cio         the strict lower triangle and the diagonal unaltered
Cio  ai    :contains information about the unitary transformations used
Cio         in the reduction (in full upper triangle)
Cio         the strict lower triangle is unaltered
Co   d     :contains the diagonal elements of the the tridiagonal matrix
Co   e     :contains the subdiagonal elements of the tridiagonal matrix
Co          in its last n-1 positions.  e(1) is set to zero.
Co   e2    :contains the squares of the corresponding elements of e
Cr          e2 may coincide with e if the squares are not needed.
Co   tau   :contains further information about the transformations
Cr Remarks:
Cr     this subroutine is a translation of a complex analogue of
Cr     the algol procedure tred1, num. math. 11, 181-195(1968)
Cr     by martin, reinsch, and wilkinson.
Cr     handbook for auto. comp., vol.ii-linear algebra, 212-226(1971).
Cr     this version is adapted from the eispack htridi. In contrast
Cr     only the upper triangle of a is used for input and output
Cr     questions and comments should be directed to burton s. garbow,
Cr     mathematics and computer science div, argonne national laboratory
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer n
      double precision ar(n,n),ai(n,n),d(n),e(n),e2(n),tau(2,n)
C Local variables:
      integer i,j,k,im1,jp1
      double precision f,g,h,fi,gi,hh,si,scale,dcabs2
C External calls:
      external  dcabs2

      tau(1,n) = 1.0d0
      tau(2,n) = 0.0d0
C
      do i = 1, n
        d(i) = ar(i,i)
      enddo

      do i = n,1,-1
         im1 = i - 1
         h = 0.0d0
         scale = 0.0d0
         if (im1.ge.1) then
C -------- scale row
           do k = 1, im1
             scale = scale + dabs(ar(k,i)) + dabs(ai(k,i))
           enddo
           if (scale .ne. 0.0d0) goto 140
           tau(1,im1) = 1.0d0
           tau(2,im1) = 0.0d0
         endif
         e (i) = 0.0d0
         e2(i) = 0.0d0
         goto 290
C
  140    do k = 1, im1
           ar(k,i) = ar(k,i) / scale
           ai(k,i) = ai(k,i) / scale
           h = h + ar(k,i) * ar(k,i) + ai(k,i) * ai(k,i)
         enddo

         e2(i) = scale * scale * h
         g = dsqrt(h)
         e(i) = scale * g
         f = dcabs2(ar(im1,i),ai(im1,i))
C ------- form next diagonal element of matrix t
         if (f .ne. 0.0d0) then
           tau(1,im1) = (ai(im1,i)*tau(2,i) - ar(im1,i)*tau(1,i)) / f
           si         = (ar(im1,i)*tau(2,i) + ai(im1,i)*tau(1,i)) / f
           h = h + f * g
           g = 1.0d0 + g / f
           ar(im1,i) = g * ar(im1,i)
           ai(im1,i) = g * ai(im1,i)
           if (im1 .eq. 1) goto 270
         else
           tau(1,im1) = -tau(1,i)
           si       =    tau(2,i)
           ar(im1,i) = g
         endif
         f = 0.0d0

         do j = 1, im1
            g  = 0.0d0
            gi = 0.0d0
C --------- form element of a*u
            do k = 1, j
               g  = g  + ar(k,j) * ar(k,i) + ai(k,j) * ai(k,i)
               gi = gi - ar(k,j) * ai(k,i) + ai(k,j) * ar(k,i)
            enddo
            jp1 = j + 1
            if (im1 .ge. jp1) then
              do  k = jp1, im1
                 g  = g  + ar(j,k) * ar(k,i) - ai(j,k) * ai(k,i)
                 gi = gi - ar(j,k) * ai(k,i) - ai(j,k) * ar(k,i)
              enddo
            endif
C --------- form element of p
            e(j)     = g  / h
            tau(2,j) = gi / h
            f = f + e(j) * ar(j,i) - tau(2,j) * ai(j,i)
         enddo

         hh = f / (h + h)
C ------ form reduced a
         do  j = 1, im1
            f = ar(j,i)
            g = e(j) - hh * f
            e(j) = g
            fi = -ai(j,i)
            gi = tau(2,j) - hh * fi
            tau(2,j) = -gi
            do k = 1, j
              ar(k,j) = ar(k,j) - f * e(k) - g * ar(k,i)
     .                    + fi * tau(2,k) + gi * ai(k,i)
              ai(k,j) = ai(k,j) - f * tau(2,k) - g * ai(k,i)
     .                            - fi * e(k) - gi * ar(k,i)
            enddo
         enddo

  270    continue
         do k = 1, im1
            ar(k,i) = scale * ar(k,i)
            ai(k,i) = scale * ai(k,i)
         enddo

         tau(2,im1) = -si
  290    hh = d(i)
         d(i) = ar(i,i)
         ar(i,i) = hh
         ai(i,i) = scale * dsqrt(h)
      enddo
      end



      double precision function dcabs2(x1,x2)
C- gets norm of complex number
C ----------------------------------------------------------------------
Ci Inputs:
Ci   x1    :real part
Ci   x2    :imaginary part
Co Outputs:
Co   dcabs2:norm of complex number
Cr Remarks:
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      double precision x1,x2

      dcabs2 = dsqrt(x1*x1 + x2*x2)
      end


      subroutine dcopy(n,dx,incx,dy,incy)
C- Copies a vector, x, to a vector, y. ( y := x)
C ----------------------------------------------------------------------
Ci Inputs:
Ci   n     :lenght of dx and dy
Ci   dx    :vector to copy
Ci   incx  :incrementation for x
Ci   dy    :vector
Ci   incy  :incrementation for y
Co Outputs:
Co   dy    :vector
Cr Remarks:
cr    Adapted from: jack dongarra, linpack, 3/11/78.
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      double precision dx(*),dy(*)
      integer incx,incy,n
C Local variables:
      integer i,ix,iy,m,mp1
c
      if(n.le.0)return
      if(incx.ne.1.or.incy.ne.1) then
c ----  code for unequal increments or equal increments not equal to 1
        ix = 1
        iy = 1
        if(incx.lt.0)ix = (-n+1)*incx + 1
        if(incy.lt.0)iy = (-n+1)*incy + 1
        do i = 1,n
          dy(iy) = dx(ix)
          ix = ix + incx
          iy = iy + incy
        enddo
      else
c ----  code for both increments equal to 1
        m = mod(n,7)
        if( m .ne. 0 ) then
          do i = 1,m
            dy(i) = dx(i)
          enddo
          if( n .lt. 7 ) return
        endif
        mp1 = m + 1
        do i = mp1,n,7
          dy(i) = dx(i)
          dy(i + 1) = dx(i + 1)
          dy(i + 2) = dx(i + 2)
          dy(i + 3) = dx(i + 3)
          dy(i + 4) = dx(i + 4)
          dy(i + 5) = dx(i + 5)
          dy(i + 6) = dx(i + 6)
        enddo
        return
      endif
      end



      subroutine dinit(array,leng)
C- Initializes double precision array to zero
C ----------------------------------------------------------------------
Ci Inputs:
Ci   leng  :length of array
Co Outputs:
Co   array :double precision array set to zero
Cr Remarks:
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer leng
      double precision array(leng)
C Local variables:
      integer i,m,mp1

      m = mod(leng,5)
      if( m .ne. 0 ) then
        do i = 1,m
          array(i) = 0.d0
        enddo
        if( leng .lt. 5 ) return
      endif
      mp1 = m + 1
      do i = mp1,leng,5
        array(i) = 0.d0
        array(i + 1) = 0.d0
        array(i + 2) = 0.d0
        array(i + 3) = 0.d0
        array(i + 4) = 0.d0
      enddo
      end



      subroutine imtql2(nm,n,d,e,z,ierr)
c
      integer i,j,k,l,m,n,ii,nm,mml,ierr
      double precision d(n),e(n),z(nm,n)
      double precision b,c,f,g,p,r,s,tst1,tst2,dcabs2
c
c     this subroutine is a translation of the algol procedure imtql2,
c     num. math. 12, 377-383(1968) by martin and wilkinson,
c     as modified in num. math. 15, 450(1970) by dubrulle.
c     handbook for auto. comp., vol.ii-linear algebra, 241-248(1971).
c
c     this subroutine finds the eigenvalues and eigenvectors
c     of a symmetric tridiagonal matrix by the implicit ql method.
c     the eigenvectors of a full symmetric matrix can also
c     be found if  tred2  has been used to reduce this
c     full matrix to tridiagonal form.
c
c     on input
c
c        nm must be set to the row dimension of two-dimensional
c          array parameters as declared in the calling program
c          dimension statement.
c
c        n is the order of the matrix.
c
c        d contains the diagonal elements of the input matrix.
c
c        e contains the subdiagonal elements of the input matrix
c          in its last n-1 positions.  e(1) is arbitrary.
c
c        z contains the transformation matrix produced in the
c          reduction by  tred2, if performed.  if the eigenvectors
c          of the tridiagonal matrix are desired, z must contain
c          the identity matrix.
c
c      on output
c
c        d contains the eigenvalues in ascending order.  if an
c          error exit is made, the eigenvalues are correct but
c          unordered for indices 1,2,...,ierr-1.
c
c        e has been destroyed.
c
c        z contains orthonormal eigenvectors of the symmetric
c          tridiagonal (or full) matrix.  if an error exit is made,
c          z contains the eigenvectors associated with the stored
c          eigenvalues.
c
c        ierr is set to
c          zero       for normal return,
c          j          if the j-th eigenvalue has not been
c                     determined after 30 iterations.
c
c     calls dcabs2 for  dsqrt(a*a + b*b) .
c
c     questions and comments should be directed to burton s. garbow,
c     mathematics and computer science div, argonne national laboratory
c
c     this version dated august 1983.
c
c     -----------------------------------------------------------------
c
      ierr = 0
      if (n .eq. 1) go to 1001
c
      do 100 i = 2, n
  100 e(i-1) = e(i)
c
      e(n) = 0.0d0
c
      do 240 l = 1, n
         j = 0
c     .......... look for small sub-diagonal element ..........
  105    do 110 m = l, n
            if (m .eq. n) go to 120
            tst1 = dabs(d(m)) + dabs(d(m+1))
            tst2 = tst1 + dabs(e(m))
            if (tst2 .eq. tst1) go to 120
  110    continue
c
  120    p = d(l)
         if (m .eq. l) go to 240
         if (j .eq. 30) go to 1000
         j = j + 1
c     .......... form shift ..........
         g = (d(l+1) - p) / (2.0d0 * e(l))
         r = dcabs2(g,1.0d0)
         g = d(m) - p + e(l) / (g + dsign(r,g))
         s = 1.0d0
         c = 1.0d0
         p = 0.0d0
         mml = m - l
c     .......... for i=m-1 step -1 until l do -- ..........
         do 200 ii = 1, mml
            i = m - ii
            f = s * e(i)
            b = c * e(i)
            r = dcabs2(f,g)
            e(i+1) = r
            if (r .eq. 0.0d0) go to 210
            s = f / r
            c = g / r
            g = d(i+1) - p
            r = (d(i) - g) * s + 2.0d0 * c * b
            p = s * r
            d(i+1) = g + p
            g = c * r - b
c     .......... form vector ..........
            do 180 k = 1, n
               f = z(k,i+1)
               z(k,i+1) = s * z(k,i) + c * f
               z(k,i) = c * z(k,i) - s * f
  180       continue
c
  200    continue
c
         d(l) = d(l) - p
         e(l) = g
         e(m) = 0.0d0
         go to 105
c     .......... recover from underflow ..........
  210    d(i+1) = d(i+1) - p
         e(m) = 0.0d0
         go to 105
  240 continue
c     .......... order eigenvalues and eigenvectors ..........
      do 300 ii = 2, n
         i = ii - 1
         k = i
         p = d(i)
c
         do 260 j = ii, n
            if (d(j) .ge. p) go to 260
            k = j
            p = d(j)
  260    continue
c
         if (k .eq. i) go to 300
         d(k) = d(i)
         d(i) = p
c
         do 280 j = 1, n
            p = z(j,i)
            z(j,i) = z(j,k)
            z(j,k) = p
  280    continue
c
  300 continue
c
      go to 1001
c     .......... set error -- no convergence to an
c                eigenvalue after 30 iterations ..........
 1000 ierr = l
 1001 return
      end
      subroutine imtqlv(n,d,e,e2,w,ind,ierr,rv1)
C- Finds the eigenvalues of a symmetric tridiagonal matrix by the
C- implicit QL method
C ----------------------------------------------------------------------
Ci Inputs:
Ci   n     :order of matrix
Ci   d     :diagonal elements of matrix
Ci   e     :subdiagonal elements of matrix in its last n-1 positions
Ci   e2    :squares of the corresponding elements of e
Co Outputs:
Cio  e2    :neglegible elements have been replaced by zero
Ci   w     :eigenvalues in ascending order
Ci   ind   :index to submatrix
Ci   ierr  :nonzero (j) if the j-th eigenvalue has not been determined
Ci          after 30 iterations
Ci   rv1   :work file
Cr Remarks:
Cr    THIS ROUTINE IS A VARIANT OF  IMTQL1  WHICH IS A TRANSLATION OF
Cr    ALGOL PROCEDURE IMTQL1, NUM. MATH. 12, 377-383(1968) BY MARTIN AND
Cr    WILKINSON, AS MODIFIED IN NUM. MATH. 15, 450(1970) BY DUBRULLE.
Cr    HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 241-248(1971).
Cr    QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
Cr    MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
Cr    THIS VERSION DATED AUGUST 1983.
Cr
Cr    IND CONTAINS THE SUBMATRIX INDICES ASSOCIATED WITH THE
Cr    CORRESPONDING EIGENVALUES IN W -- 1 FOR EIGENVALUES
Cr    BELONGING TO THE FIRST SUBMATRIX FROM THE TOP,
Cr    2 FOR THOSE BELONGING TO THE SECOND SUBMATRIX, ETC..
Cr
Cr    IF AN ERROR EXIT IS MADE, THE EIGENVALUES ARE CORRECT AND
Cr    ORDERED FOR INDICES 1,2,...IERR-1, BUT MAY NOT BE
Cr    THE SMALLEST EIGENVALUES.
C ----------------------------------------------------------------------
      implicit none
C Passed variables:
      integer ierr,ind(*),n
      double precision d(n),e(n),e2(n),w(n),rv1(n)
C Local variables:
      integer i,j,k,l,m,ii,mml,tag
      double precision b,c,f,g,p,r,s,tst1,tst2,dcabs2
C External calls:
      external  dcabs2
C
      ierr = 0
      k = 0
      tag = 0
C
      do 100 i = 1, n
         w(i) = d(i)
         if (i .ne. 1) rv1(i-1) = e(i)
  100 continue
C
      e2(1) = 0.0d0
      rv1(n) = 0.0d0
C
      do 290 l = 1, n
         j = 0
C     .......... LOOK FOR SMALL SUB-DIAGONAL ELEMENT ..........
  105    do 110 m = l, n
            if (m .eq. n) goto 120
            tst1 = dabs(w(m)) + dabs(w(m+1))
            tst2 = tst1 + dabs(rv1(m))
            if (tst2 .eq. tst1) goto 120
C     .......... GUARD AGAINST UNDERFLOWED ELEMENT OF E2 ..........
            if (e2(m+1) .eq. 0.0d0) goto 125
  110    continue
C
  120    if (m .le. k) goto 130
         if (m .ne. n) e2(m+1) = 0.0d0
  125    k = m
         tag = tag + 1
  130    p = w(l)
         if (m .eq. l) goto 215
         if (j .eq. 30) goto 1000
         j = j + 1
C     .......... FORM SHIFT ..........
         g = (w(l+1) - p) / (2.0d0 * rv1(l))
         r = dcabs2(g,1.0d0)
         g = w(m) - p + rv1(l) / (g + dsign(r,g))
         s = 1.0d0
         c = 1.0d0
         p = 0.0d0
         mml = m - l
C     .......... FOR I=M-1 STEP -1 UNTIL L DO -- ..........
         do 200 ii = 1, mml
            i = m - ii
            f = s * rv1(i)
            b = c * rv1(i)
            r = dcabs2(f,g)
            rv1(i+1) = r
            if (r .eq. 0.0d0) goto 210
            s = f / r
            c = g / r
            g = w(i+1) - p
            r = (w(i) - g) * s + 2.0d0 * c * b
            p = s * r
            w(i+1) = g + p
            g = c * r - b
  200    continue
C
         w(l) = w(l) - p
         rv1(l) = g
         rv1(m) = 0.0d0
         goto 105
C     .......... RECOVER FROM UNDERFLOW ..........
  210    w(i+1) = w(i+1) - p
         rv1(m) = 0.0d0
         goto 105
C     .......... ORDER EIGENVALUES ..........
  215    if (l .eq. 1) goto 250
C     .......... FOR I=L STEP -1 UNTIL 2 DO -- ..........
         do 230 ii = 2, l
            i = l + 2 - ii
            if (p .ge. w(i-1)) goto 270
            w(i) = w(i-1)
            ind(i) = ind(i-1)
  230    continue
C
  250    i = 1
  270    w(i) = p
         ind(i) = tag
  290 continue
C
      goto 1001
C     .......... SET ERROR -- NO CONVERGENCE TO AN
C                EIGENVALUE AFTER 30 ITERATIONS ..........
 1000 ierr = l
 1001 return
      end
