	SUBROUTINE VGENSPH(Rmax,Nmax,A,B,C,R,N,Vect,Nco,Rco,Lastg,Inf)
*****************************************************************
*       Subroutine generates all vectors contained              *
*       in sphere Rmax but not larger than Nmax.                *
*                                                               *
*       Input:                                                  *
*               Rmax,Nmax  - limiting parameters                *
*               A,B,C      - translations                       *
*       Output:                                                 *
*               R,N        - sphere radii and number of all     *
*                            vectors in the sphere              *
*               Vect       - vectors
*               Nco,Rco(n) - number of co-sphers and there      *
*                            radii !R=Rco(Nco)                  *
*               Lastg(n)   - number of last vector in           *
*                            n-th co-sphere.                    *
*               Inf        - shows generation way: 0 - Rmax,    *
*                            1 - Nmax. Currently does not work! *
*****************************************************************
	INCLUDE 'PARAM.DAT'
	parameter (ncomax=nvcmax/2)
	implicit real*8 (a-h,o-z)
c
	integer lastg(ncomax),iref(nvcmax)
	dimension a(3),b(3),c(3),vec(3,nvcmax),rco(ncomax)
     &   ,rw(nvcmax),vect(3,0:nvcmax),vt(3)
	data pi/3.141592653589793d0/
c volume per site:
	vs=a(1)*(b(2)*c(3)-b(3)*c(2))+a(2)*(b(3)*c(1)-b(1)*c(3))+
     &    a(3)*(b(1)*c(2)-b(2)*c(1))
	vs=abs(vs)
c select rrab:
	rrab=rmax
	inf=0
c generate all vectors in rrab sphere:
	aa=sqrt(a(1)*a(1)+a(2)*a(2)+a(3)*a(3))
	bb=sqrt(b(1)*b(1)+b(2)*b(2)+b(3)*b(3))
	cc=sqrt(c(1)*c(1)+c(2)*c(2)+c(3)*c(3))
	l1=rrab/vs*bb*cc+1
	l2=rrab/vs*aa*cc+1
	l3=rrab/vs*aa*bb+1
c
		nc=0
		do 30 i=-l1,l1
		do 20 j=-l2,l2
		do 10 k=-l3,l3
		vt(1)=dble(i)*a(1)+dble(j)*b(1)+dble(k)*c(1)
		vt(2)=dble(i)*a(2)+dble(j)*b(2)+dble(k)*c(2)
		vt(3)=dble(i)*a(3)+dble(j)*b(3)+dble(k)*c(3)
		rvt=sqrt(vt(1)**2+vt(2)**2+vt(3)**2)
		if(rvt.gt.rrab+1.d-08)goto10
		nc=nc+1
	if(nc.gt.nvcmax)stop'<VGENSPH>: INCREASE NVCMAX IN PARAM.DAT'
		vec(1,nc)=vt(1)
		vec(2,nc)=vt(2)
		vec(3,nc)=vt(3)
		rw(nc)=rvt
  10            continue
  20            continue
  30            continue
	if(nc.eq.1)then
	nco=1
	n=0
	r=0.d0
	rco(1)=0.D0
	lastg(1)=0
	vect(1,0)=0.D0
	vect(2,0)=0.D0
	vect(3,0)=0.D0
	return
	endif
c ordering:
	call XORDER(nc,rw,nco,rco,iref)
c filling co-spheres:
c
	nvec=0
	do 60 i=2,nco
	rco(i-1)=rco(i)
	do 50 j=1,nc
	if(iref(j).eq.i) then
	nvec=nvec+1
	do 40 k=1,3
	vect(k,nvec)=vec(k,j)
  40    continue
	endif
  50    continue
	lastg(i-1)=nvec
	if(nvec.gt.nmax) goto 70
  60    continue
c output:
	nco=nco-1
	n=nvec
	goto 80
  70    nco=i-2
	n=lastg(i-2)
  80    r=rco(nco)
	vect(1,0)=0.d0
	vect(2,0)=0.d0
	vect(3,0)=0.d0
	return
	end

	



	SUBROUTINE SITEGEN(Rmax,A,B,C,Tau,Natom,Ns,Site,Iref)   
*****************************************************************
*       Subroutine for generation of all sites contained        *
*       in sphere Rmax.                                         *
*                                                               *
*       Input:                                                  *
*               A,B,C      - translations                       *
*               Tau,Natom  - basis                              *
*       Output:                                                 *
*               Ns         - number of all sites in the sphere  *
*               Site       - vectors                            *
*               Iref(n)    - shows the site sort                *
*****************************************************************
	INCLUDE 'PARAM.DAT'
	parameter (ncomax=NVCMAX/2)
	implicit real*8 (a-h,o-z)
c
	integer   lastg(ncomax),iref(nvcmax)
	dimension a(3),b(3),c(3),vect(3,0:nvcmax),rco(ncomax),
     &            site(3,nvcmax),tau(3,npat),tmps(3),
     &            tauc(3,npat)
	data pi/3.141592653589793d0/
c
	radd=0.d0
	do 10 i=1,natom
	call reduc(a,b,c,tau(1,i),tauc(1,i),0)
	s=sqrt(tauc(1,i)**2+tauc(2,i)**2+tauc(3,i)**2)
	radd=max(radd,s)
  10    continue
c select rrab:
	rrab=rmax+radd
c generation of all vectors in rrab sphere:
	call vgensph(rrab,NVCMAX,a,b,c,rou,nvec,vect,nco,rco,lastg,inf)
	if(inf.eq.1) print*,'WARRNING
     &  FROM SITEGEN: some sites may be missed'
c
	ns=0
	do 40 i=0,nvec
	do 30 j=1,natom
	tmps(1)=vect(1,i)+tauc(1,j)
	tmps(2)=vect(2,i)+tauc(2,j)
	tmps(3)=vect(3,i)+tauc(3,j)
	sch=sqrt(tmps(1)**2+tmps(2)**2+tmps(3)**2)
	if((sch-rmax).gt.1.d-08)goto30
	ns=ns+1
	if(ns.gt.nvcmax)stop'<SITEGEN>: SUBSCRIPT OUT OF BOUNDS,
     &  INCREASE NVCMAX IN PARAM.DAT'
	iref(ns)=j
	site(1,ns)=tmps(1)
	site(2,ns)=tmps(2)
	site(3,ns)=tmps(3)
  30    continue
  40    continue
	return
	end

	

	subroutine XORDER(Nin,A,Nout,B,Iref)    
**********************************************************
*       SUBROUTINE FOR ORDERING AREA WITH LARGE          *
*           NUMBER OF ELEMENTS BY DECREASING             *
*                                                        *
*       INPUT: Nin  -number of elements,A(i=1,Nin)-area. *
*      OUTPUT: Nout -number of non eqvivalent elements,  *
*              B(i=1,Nout)-ordering area.                *
**********************************************************
	integer iref(Nin)
	real*8 a(Nin),b(Nin),anext
c start up:
	b(1)=a(1)
	iref(1)=1
	do 1 i=1,Nin
	if(a(i).eq.b(1)) then
	iref(i)=1
	goto 1
	endif
	if(a(i).gt.b(1)) then
	b(2)=a(i)
	iref(i)=2
	goto 2
	else
	b(1)=a(i)
	b(2)=a(1)
		do 10 k=1,i-1
		iref(k)=2
  10            continue
		iref(i)=1
	goto 2
	endif
   1    continue
	Nout=1
	return
c
   2    Nstart=i+1
	Nout=2
c
c ordering:
	do 60 i=Nstart,Nin
	anext=a(i)
	ndown=1
	nup=Nout
c check boundary points:
		if(anext.le.b(ndown)) then
			if(anext.eq.b(ndown)) then
			iref(i)=ndown
			goto 60
			endif
		do 20 j=Nout,1,-1
		b(j+1)=b(j)
  20            continue
		do 30 k=1,i-1
		iref(k)=iref(k)+1
  30            continue
		b(1)=anext
		iref(i)=1
		Nout=Nout+1
		goto 60
		endif
		if(anext.ge.b(nup)) then
			if(anext.eq.b(nup)) then
			iref(i)=nup
			goto 60
			endif   
		Nout=Nout+1
		b(Nout)=anext
		iref(i)=Nout
		goto 60
		endif
c searching of place for anext:
   3    nd=nup-ndown
	if (nd.eq.1) then
	do 40 j=Nout,nup,-1
	b(j+1)=b(j)
  40    continue
	do 50 k=1,i-1
	if(iref(k).ge.nup) iref(k)=iref(k)+1
  50    continue
	b(nup)=anext
	iref(i)=nup
	Nout=Nout+1
	goto 60
	endif
c
	nmid=ndown+nd/2
	if(anext.eq.b(nmid)) then
	iref(i)=nmid
	goto 60
	endif
	if(anext.gt.b(nmid)) then
	ndown=nmid
	else
	nup=nmid
	endif
	goto 3
  60    continue
	return
	end                                     


	subroutine reduc(a,b,c,rin,rout,key)
*************************************************************
*       Subroutine for reducing vector R in first (Key=1) , *
*       central (Key=0) lattice cell.If (key=-1) return the *
*       R in basis coordinate.                              *
*************************************************************
	implicit real*8 (a-h,o-z)
	dimension rin(3),rout(3),rw(3),a(3),b(3),c(3),abc(3,3)
     &    ,abci(3,3)
	data icall/1/
c
c start up:
c       if(icall.eq.1) then
	icall=2
	do 10 i=1,3
	abc(i,1)=a(i)
	abc(i,2)=b(i)
	abc(i,3)=c(i)
  10    continue
	abci(1,1)=b(2)*c(3)-b(3)*c(2)
	abci(1,2)=b(3)*c(1)-b(1)*c(3)
	abci(1,3)=b(1)*c(2)-b(2)*c(1)
	det=a(1)*abci(1,1)+a(2)*abci(1,2)+a(3)*abci(1,3)
	if (abs(det).le.1.d-08) PRINT*,'invalid basis in REDUC'
	abci(2,1)=a(3)*c(2)-a(2)*c(3)
	abci(2,2)=a(1)*c(3)-a(3)*c(1)
	abci(2,3)=a(2)*c(1)-a(1)*c(2)
	abci(3,1)=a(2)*b(3)-a(3)*b(2)
	abci(3,2)=a(3)*b(1)-a(1)*b(3)
	abci(3,3)=a(1)*b(2)-a(2)*b(1)
	do 30 i=1,3
	do 20 j=1,3
	abci(i,j)=abci(i,j)/det
  20    continue
  30    continue
c       endif
c
c Rin in a,b,c basis:
	do 50 i=1,3
	s=0
	do 40 j=1,3
	s=s+abci(i,j)*rin(j)
  40    continue
	rout(i)=s
  50    continue
	if(key.lt.0) return
c
	rw(1)=rout(1)-aint(rout(1))
	rw(2)=rout(2)-aint(rout(2))
	rw(3)=rout(3)-aint(rout(3))
c first cell:
	if(rw(1).lt.0.) rw(1)=rw(1)+1
	if(rw(2).lt.0.) rw(2)=rw(2)+1
	if(rw(3).lt.0.) rw(3)=rw(3)+1
c return to old basis:
	do 70 i=1,3
	s=0
	do 60 j=1,3
	s=s+abc(i,j)*rw(j)
  60    continue
	rout(i)=s
  70    continue
	if(key.eq.1) return
c central cell:
	i0=0
	j0=0
	k0=0
	rc=rout(1)**2+rout(2)**2+rout(3)**2
	do 160 i=0,1
	do 150 j=0,1
	do 140 k=0,1
		rmod=0
		do 170 l=1,3
	 rmod=rmod+(rout(l)-i*a(l)-j*b(l)-k*c(l))**2
 170            continue
	if(rmod.lt.rc) then
	i0=i
	j0=j
	k0=k
	rc=rmod
	endif
 140    continue
 150    continue
 160    continue
		do 180 l=1,3
	 rout(l)=rout(l)-i0*a(l)-j0*b(l)-k0*c(l)
 180            continue
	return
	end
		
	Subroutine NASOS(wr,rmt,a,b,c,tau,natom)
*********************************************************
*       Subroutine for determining MT-sphere            *
*       radii according to  input weight.               *
*********************************************************
	INCLUDE 'PARAM.DAT'
	implicit real*8 (a-h,o-z)
	integer   isat(nvcmax),flag(npat)
	dimension a(3),b(3),c(3),site(3,nvcmax),tauc(3,npat)
     &           ,wr(natom),rmt(natom),tau(3,natom)
c
c generation all sites in 8 cell:
	do 10 i=1,natom
	call reduc(a,b,c,tau(1,i),tauc(1,i),1)
  10    continue
	ns=0
	do 60 i=0,1
	do 50 j=0,1
	do 40 k=0,1
	do 30 iat=1,natom
	ns=ns+1
	if(ns.gt.nvcmax)stop'<strpac>: increase nvcmax in param.dat'
	isat(ns)=iat
	 do 20 l=1,3
	 site(l,ns)=tauc(l,iat)-i*a(l)-j*b(l)-k*c(l)
  20     continue
  30    continue
  40    continue
  50    continue
  60    continue
c basis set:
	do 70 i=1,natom
	call reduc(a,b,c,tau(1,i),tauc(1,i),0)
  70    continue
	do 110 iat=1,natom
		do 100 i=1,ns
		rw=(site(1,i)-tauc(1,iat))**2+
     &             (site(2,i)-tauc(2,iat))**2+
     &             (site(3,i)-tauc(3,iat))**2
		if(rw.lt.1.d-08) then
		do 80 k=1,3
		site(k,i)=site(k,iat)
  80            continue
		do 90 k=1,3
		site(k,iat)=tauc(k,iat)
  90            continue
		goto 110
		endif
 100            continue
 110    continue
c start up:
	do 120 i=1,natom
	rmt(i)=0.
	flag(i)=0
 120    continue
c blowing coefficient:
		dog=0
 125    al0=1.d10
	       dog=dog+1
	       if (dog.gt.100)
     &  PRINT*,'STOP FROM NASOS: IMPLICIT CYCLE ABORT'
	do 140 i=1,natom
	if(flag(i).eq.1) goto 140
	do 130 j=1,ns
	if(i.eq.j) goto 130
	dij=sqrt((site(1,i)-site(1,j))**2+(site(2,i)-site(2,j))**2+
     &          (site(3,i)-site(3,j))**2)
	al=(dij-rmt(isat(j))*flag(isat(j)))/
     &    (wr(i)+wr(isat(j))*(1-flag(isat(j))))
	al0=min(al0,al)
 130    continue
 140    continue
c blowing:
		do 150 i=1,natom
		if(flag(i).eq.0) rmt(i)=wr(i)*al0
 150            continue
c flag set:
	do 170 i=1,natom
	do 160 j=1,ns
	if(i.eq.j) goto 160
	dij=sqrt((site(1,i)-site(1,j))**2+(site(2,i)-site(2,j))**2+
     &          (site(3,i)-site(3,j))**2)
	g=abs(dij-rmt(i)-rmt(isat(j)))
	if(g.le.1.d-08) then
	flag(i)=1
	goto 170
	endif
 160    continue
 170    continue
c check:        
	ncheck=0
		do 180 i=1,natom
		ncheck=ncheck+flag(i)
 180            continue
		if(ncheck.lt.natom) goto 125
	return
	end     




	SUBROUTINE POLIDET(NGR,POLI,WR,WS,IREFAT,RMIN,RMAX,VOL)
************************************************************
*       PROGRAM FOR CONSTRUCTING POLYHEDRON SURFACE        *
*                                                          *
*       INPUT: POLYHEDRON BY LISTING OF VECTORS POINTED AT *
*              NEAREST SITE,MT-RADII OF EACH SORT SITES    *
*       OUTPUT: PLANE NUMBER,FULL LIST OF POINTS,LIST OF   *
*               TRIANGULAR                                 *
************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	PARAMETER (NTRMAX=NENVMAX)
	INTEGER NTR(NENVMAX),LISTGR(NTRMAX),LISTTR(3,NTRMAX,NENVMAX)
     &   ,IBAD(NENVMAX),IREFAT(0:NVCMAX)
	DIMENSION POLI(3,NVCMAX),POLIST(3,NVCMAX),PP(3),WR(0:NVCMAX)
     &   ,XW(3),YW(3),ZW(3),WRD(3),WS(NVCMAX)
	COMMON/STORE1/ NP0,POLIST,NTR,LISTTR
C
	DO 10 I=1,NGR
	DIJ=POLI(1,I)**2+POLI(2,I)**2+POLI(3,I)**2
	WS(I)=.5D0+(WR(0)**2-WR(I)**2)/2./DIJ
  10    CONTINUE
C -------------- VERTEX SEARCHING -----------
	NP0=0
	DO 50 I1=1,NGR-2
	WRD(1)=WS(I1)
	DO 50 I2=I1+1,NGR-1
	WRD(2)=WS(I2)
	DO 50 I3=I2+1,NGR
	WRD(3)=WS(I3)
	CALL SYS3(POLI(1,I1),POLI(1,I2),POLI(1,I3),WRD,PP,DET,IERR)
		IF(IERR.EQ.0) THEN
C -------------- VERTEX CHECKING ------------
		DO 20 IV=1,NGR
		DP=PP(1)*POLI(1,IV)+PP(2)*POLI(2,IV)+
     &             PP(3)*POLI(3,IV)
		DR=(POLI(1,IV)**2+POLI(2,IV)**2+POLI(3,IV)**2+
     &              WR(0)**2-WR(IV)**2)/2.D0+1.D-08
		IF(DP.GE.DR) GOTO 50
  20            CONTINUE
			IF(NP0.EQ.0)  GOTO 35
		DO 30 IP=1,NP0
		DP=SQRT((POLIST(1,IP)-PP(1))**2+(POLIST(2,IP)-PP(2))**2+
     &                  (POLIST(3,IP)-PP(3))**2)
		IF(DP.LE.1.D-08) GOTO 50
  30            CONTINUE
  35            NP0=NP0+1
		DO 40 I=1,3
		POLIST(I,NP0)=PP(I)
  40            CONTINUE
		ENDIF
  50    CONTINUE
C --------------- CLASSIFICATION ON PLANE ---------------
	NGR0=0
	DO 70 IV=1,NGR
		NTT=0
		DO 60 IP=1,NP0
		X1=POLIST(1,IP)-POLI(1,IV)*WS(IV)
		X2=POLIST(2,IP)-POLI(2,IV)*WS(IV)
		X3=POLIST(3,IP)-POLI(3,IV)*WS(IV)
		X=POLI(1,IV)*X1+POLI(2,IV)*X2+POLI(3,IV)*X3
		AX=ABS(X)
		IF (AX.LE.1.D-08) THEN
		NTT=NTT+1
		LISTGR(NTT)=IP
		ENDIF
  60            CONTINUE
	IF (NTT.GE.3) THEN
	NGR0=NGR0+1
	IF(NENVMAX.LT.NGR0)STOP'<POLYDET>:
     &  INCREASE NENVMAX IN PARAM.DAT'
	IBAD(NGR0)=IV
	NTR(NGR0)=NTT
	CALL GRANI(NTT,LISTGR,POLI(1,IV),LISTTR(1,1,NGR0))
	ENDIF
  70    CONTINUE
C ------------------- POLYHEDRON RESET -----------------
	NGR=NGR0
	IF(NGR.LE.3) PRINT*, 'STOP FROM POLYDET:
     &          NEAREST SITES SET IS NOT COMPLETE'
	DO 90 I1=1,NGR
	WS(I1)=WS(IBAD(I1))
	IREFAT(I1)=IREFAT(IBAD(I1))
		DO 80 I2=1,3
		POLI(I2,I1)=POLI(I2,IBAD(I1))
  80            CONTINUE
  90    CONTINUE
C
	RMIN=DSQRT(POLI(1,1)**2+POLI(2,1)**2+POLI(3,1)**2)*WS(1)
	DO 100 I=2,NGR
	X1=SQRT(POLI(1,I)**2+POLI(2,I)**2+POLI(3,I)**2)*WS(I)
	RMIN=MIN(RMIN,X1)
 100    CONTINUE
C
	RMAX=SQRT(POLIST(1,1)**2+POLIST(2,1)**2+POLIST(3,1)**2)
	DO 110 I=1,NP0
	X1=SQRT(POLIST(1,I)**2+POLIST(2,I)**2+POLIST(3,I)**2)
	RMAX=MAX(RMAX,X1)
 110    CONTINUE
C
C ---------- POLYHEDRON VOLUME --------
	VOL=0
	DO 140 I=1,NGR
	 XARC=SQRT(POLI(1,I)**2+POLI(2,I)**2+POLI(3,I)**2)*WS(I)
	DO 130 J=1,NTR(I)
	N01=LISTTR(1,J,I)
	N02=LISTTR(2,J,I)
	N03=LISTTR(3,J,I)
		DO 120 I1=1,3
		XW(I1)=POLIST(I1,N02)-POLIST(I1,N01)
		YW(I1)=POLIST(I1,N03)-POLIST(I1,N01)
 120            CONTINUE
	ZW(1)=XW(2)*YW(3)-XW(3)*YW(2)
	ZW(2)=XW(3)*YW(1)-XW(1)*YW(3)
	ZW(3)=XW(1)*YW(2)-XW(2)*YW(1)
	SSTR=SQRT(ZW(1)**2+ZW(2)**2+ZW(3)**2)
	VOL=VOL+SSTR*XARC/6.D0
 130    CONTINUE
 140    CONTINUE
	RETURN
	END



	SUBROUTINE GRANI(NTT,LIST,VP,LISTTR)
	IMPLICIT REAL*8 (A-H,O-Z)
	INCLUDE 'PARAM.DAT'
	PARAMETER (NTRMAX=NENVMAX)
	INTEGER LISTTR(3,NTRMAX),LIST(NTRMAX)
	DIMENSION POLIST(3,NVCMAX),VP(3),X(3),Y(3),FI(NTRMAX)
	COMMON/STORE1/ NP0,POLIST
	DATA PI/3.141592653589793D0/
C
	X(1)=0
	X(2)=0
	X(3)=0
	A1=SQRT(VP(1)**2+VP(2)**2+VP(3)**2)
C
C ------------- PLANE CENTER ------------
	DO 10 I=1,NTT
	X(1)=X(1)+POLIST(1,LIST(I))
	X(2)=X(2)+POLIST(2,LIST(I))
	X(3)=X(3)+POLIST(3,LIST(I))
  10    CONTINUE
	NP0=NP0+1
	POLIST(1,NP0)=X(1)/DBLE(NTT)
	POLIST(2,NP0)=X(2)/DBLE(NTT)
	POLIST(3,NP0)=X(3)/DBLE(NTT)
C
C ----------------- PARAMETERIZATION -----------------
	NW=LIST(1)
	DO 20 I=1,3
	X(I)=POLIST(I,NW)-POLIST(I,NP0)
  20    CONTINUE
	A2=SQRT(X(1)**2+X(2)**2+X(3)**2)
	DO 30 I=1,3
	X(I)=X(I)/A2
  30    CONTINUE
	Y(1)=(VP(2)*X(3)-VP(3)*X(2))/A1
	Y(2)=(VP(3)*X(1)-VP(1)*X(3))/A1
	Y(3)=(VP(1)*X(2)-VP(2)*X(1))/A1
	DO 40 I=1,NTT
	X0= X(1)*(POLIST(1,LIST(I))-POLIST(1,NP0))+
     &     X(2)*(POLIST(2,LIST(I))-POLIST(2,NP0))+
     &     X(3)*(POLIST(3,LIST(I))-POLIST(3,NP0))
	Y0= Y(1)*(POLIST(1,LIST(I))-POLIST(1,NP0))+
     &     Y(2)*(POLIST(2,LIST(I))-POLIST(2,NP0))+
     &     Y(3)*(POLIST(3,LIST(I))-POLIST(3,NP0))
	A3=SQRT(X0**2+Y0**2)
	X0=X0/A3
	Y0=Y0/A3
	IF(X0.GT.1.D0)X0=1.D0
	IF(Y0.GT.1.D0)Y0=1.D0
	IF(X0.LT.-1.D0)X0=-1.D0
	IF(Y0.LT.-1.D0)Y0=-1.D0
	FI0=ACOS(X0)
	FI1=ASIN(Y0)
	IF (FI1.GE.0) THEN
	FI(I)=FI0
	ELSE
	FI(I)=2.D00*PI-FI0
	ENDIF
  40    CONTINUE
C
C ----------------- ORDERING BY FI PARAMETER ------------
	DO 60 J=1,NTT
	IM=J
	FIM=FI(J)
	DO 50 I=J,NTT
	IF(FI(I).LT.FIM) THEN
	FIM=FI(I)
	IM=I
	ENDIF
  50    CONTINUE
	FF=FI(J)
	FI(J)=FI(IM)
	FI(IM)=FF
	I0=LIST(J)
	LIST(J)=LIST(IM)
	LIST(IM)=I0
  60    CONTINUE
C
C -------------- LIST OF TRIANGLES--------------
	LIST(NTT+1)=LIST(1)
	DO 70 I=1,NTT
	LISTTR(1,I)=NP0
	LISTTR(2,I)=LIST(I)
	LISTTR(3,I)=LIST(I+1)
  70    CONTINUE
	RETURN
	END     



	SUBROUTINE SYS3(A,B,C,W,X,DET,IERR)
	IMPLICIT REAL*8 (A-H,O-Z)
	DIMENSION A(3),B(3),C(3),X(3),W(3),ABCI(3,3),D(3)
C
	IERR=0
	ABCI(1,1)=B(2)*C(3)-C(2)*B(3)
	ABCI(1,2)=A(3)*C(2)-A(2)*C(3)
	ABCI(1,3)=A(2)*B(3)-B(2)*A(3)
	DET=A(1)*ABCI(1,1)+B(1)*ABCI(1,2)+C(1)*ABCI(1,3)
	ADET=ABS(DET)
		IF(ADET.LE.1.D-8) THEN
		IERR=1
		RETURN
		ENDIF
	ABCI(2,1)=B(3)*C(1)-B(1)*C(3)
	ABCI(2,2)=A(1)*C(3)-C(1)*A(3)
	ABCI(2,3)=A(3)*B(1)-A(1)*B(3)
	ABCI(3,1)=B(1)*C(2)-B(2)*C(1)
	ABCI(3,2)=A(2)*C(1)-A(1)*C(2)
	ABCI(3,3)=A(1)*B(2)-A(2)*B(1)
C
	D(1)=(A(1)**2+A(2)**2+A(3)**2)*W(1)
	D(2)=(B(1)**2+B(2)**2+B(3)**2)*W(2)
	D(3)=(C(1)**2+C(2)**2+C(3)**2)*W(3)
	DO 20 I=1,3
	S=0.
	DO 10 J=1,3
	S=S+ABCI(I,J)*D(J)
  10    CONTINUE
	X(I)=S/DET
  20    CONTINUE
	RETURN
	END



	SUBROUTINE POLISRCH(RMT,IATOM,NSRCH,NGR,RMIN
     &                                ,RMAX,VOL,RV,POLI,WS,IREFAT)
******************************************************
*          Routine for polyhedron searching          *
******************************************************
	INCLUDE 'PARAM.DAT'
	PARAMETER (NTRMAX=NENVMAX)
	IMPLICIT REAL*8 (A-H,O-Z)
c
	INTEGER   IREF(NVCMAX),ILST(NSORTMAX),ILSB(NSORTMAX)
     &           ,ILSV(NSORTMAX),IS(NPAT),IREFAT(0:NVCMAX)
	DIMENSION SITE(3,NVCMAX),TAUC(3,NPAT),WS(NVCMAX)
	DIMENSION RMT(NPAT),WT(0:NVCMAX),POLI(3,NVCMAX)
c
	COMMON /PARAM/  NSORT,IS,ILST,ILSB,ILSV,NATOM       !!! from init
	COMMON /LATTICE/A(3,3)                              !!! from init
	COMMON /BASIS/  TAU(3,NPAT)                         !!! from init
c
c start up:
		do 10 i=1,natom
		do 10 k=1,3
		tauc(k,i)=tau(k,i)-tau(k,iatom)
  10            continue
		tt=1.D+10
		do 20 i=1,natom
		tt=MIN(tt,rmt(iatom)**2-rmt(i)**2)
  20            continue
c
c searching by iteration:
c
	hdiv=1./DBLE(nsrch)
	it=0
c
  25    it=it+1
	if(it.gt.100)PRINT*,'STOP FROM <POLISRCH>: implicit cycle abort'
	ck=1.+hdiv*it
	if(ck.gt.5.)
     &  PRINT*,'error in searching polyhedron: rou/mt exeeded 5'
		rou=rmt(iatom)*ck
		rgen=rou+SQRT(rou**2-tt)
	CALL SITEGEN(rgen,a(1,1),a(1,2),a(1,3),tauc,Natom,Ns,Site,Iref)
C formation poli:
	wt(0)=rmt(iatom)
	irefat(0)=iatom
	ngr=0
	rv=1.d+10
	do 40 i=1,Ns
	dij=site(1,i)**2+site(2,i)**2+site(3,i)**2
	if (dij.le.1.D-08) goto 40
	rv0=sqrt(dij)
	rv=min(rv,rv0)
	al=.5d0+(rmt(iatom)**2-rmt(iref(i))**2)/2.d0/dij
	if (SQRT(dij)*al.gt.rou) goto 40
	ngr=ngr+1
	if(ngr.gt.nvcmax)stop'<strpac>  increase nvcmax inside 
     &  param.dat'
		do 30 k=1,3
		poli(k,ngr)=site(k,i)
  30            continue
	wt(ngr)=rmt(iref(i))
	irefat(ngr)=iref(i)
  40    continue
C
	CALL POLIDET(ngr,poli,wt,ws,irefat,rmin,rmax,vol)
	if (it.eq.1) then
	vol0=vol
	goto 25
	endif
	if (vol.eq.vol0.and.(vol.ne.0)) then
	return
	else
	vol0=vol
	goto 25
	endif
	end
